from translation_canvas.readwrite_database import write_data, read_data
import json
import os
import argparse

path_to_file = os.path.dirname(os.path.abspath(__file__))
JOBS_PATH = os.path.join(path_to_file, "jobs")
logging = False

logging = False
def add_run_to_db(run_name, src_lang, tgt_lang):
    run_id = write_data(f"INSERT INTO runs (filename, source_lang, target_lang, in_progress, run_type) VALUES ('{run_name}', '{src_lang}', '{tgt_lang}', 0, 'none'); SELECT id FROM runs ORDER BY id DESC LIMIT 1")[0][0]
    pairs=json.load(open(os.path.join(JOBS_PATH, run_name, f"{run_name}_extracted.json"), 'r'))
    length = len(pairs)
    for i in range(length):
        reference = pairs[i]['reference'].replace("'", "''")
        prediction = pairs[i]['prediction'].replace("'", "''")
        results = read_data(f"SELECT id FROM refs WHERE source_text = '{reference}';", logging=logging)
        if results == []:
            results = write_data(f"INSERT INTO refs (source_text, lang) VALUES ('{reference}', '{tgt_lang}'); SELECT id FROM refs ORDER BY id DESC LIMIT 1;", logging=logging)
        ref_id = results[0][0]
        write_data(f"INSERT INTO preds (source_text, ref_id, run_id) VALUES ('{prediction}', {ref_id}, {run_id}); INSERT INTO preds_text (source_text, pred_id) VALUES ('{prediction}', (SELECT id FROM preds ORDER BY id DESC LIMIT 1));", logging=logging)[0][0]
        results = write_data(f"", logging=logging)
        if i % 100 == 0:
            write_data(f"UPDATE runs SET num_predictions = {i},in_progress = {i/length} WHERE id = {run_id};", logging=logging)
    results = write_data(f"UPDATE runs SET num_predictions = {length}, in_progress = 1, exit_status=0 WHERE id = {run_id};", logging=logging)

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--run_name', type=str, required=True)
    parser.add_argument('--src_lang', type=str, required=True)
    parser.add_argument('--tgt_lang', type=str, required=True)
    args = parser.parse_args()

    add_run_to_db(args.run_name, args.src_lang, args.tgt_lang)


import json
import os
path_to_file = os.path.dirname(os.path.abspath(__file__))


def extract_pairs_from_csv(files: list[str]) -> list:
    """
    Extracts source-prediction-reference tuples (if available) from a .csv file and creates a dictionairy containing those pairs
   
    Args:
        files (list[str]): A list of file paths to the inputted csv files.

    Returns:
        pairs (list): A list of dictionaries, each containing a reference and a prediction.

    Raises:
        FileNotFoundError: If the specified log file does not exist.

    """
    pairs = []                # Holds the data that has been read in

    with open(files[0]) as f:
        lines = f.readlines()   
        for line in lines:
            data = json.loads(line)     # Load the data from the line
            pairs.append({"prediction": data["prediction"], "reference": data["reference"]})
            # Add only prediction and reference data to the pairs list
    return pairs


"""
After running this script, if it ran correctly, the output should look like (assuming source and reference 
included in the csv file):
[
    {
        source: "source1",
        prediction: "prediction1",
        reference: "reference1"
    },
    {
        source: "source2",
        prediction: "prediction2",
        reference: "reference2"
    },
    {
        source: "source3",
        prediction: "prediction3",
        reference: "reference3"
    },
    ...
]
"""


import torch
from typing import Dict, TypeVar, Iterable, List
import transformers
from transformers import LlamaForCausalLM, LlamaTokenizer, AutoModelForCausalLM, AutoTokenizer
from tqdm import tqdm

T = TypeVar('T')

DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"


def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    tokenizer.add_special_tokens(special_tokens_dict)
    tokenizer.add_special_tokens(
        {
            "eos_token": DEFAULT_EOS_TOKEN,
            "bos_token": DEFAULT_BOS_TOKEN,
            "unk_token": DEFAULT_UNK_TOKEN,
        }
    )


class InstructScore:
    def __init__(self, batch_size=2, max_src_len=512, max_trg_len=512, device_id="cuda", task_type="mt_zh-en", cache_dir=None):
        self.batch_size = batch_size
        self.max_src_len = max_src_len
        self.max_trg_len = max_trg_len
        self.device_id = device_id
        self.task_type = task_type
        print("Max source length: ", max_src_len)
        print("MAX target length: ", max_trg_len)

        print("Loading InstructScore model and tokenizer... ")
        if task_type == 'mt_zh-en':
            self.tokenizer = LlamaTokenizer.from_pretrained(
                "xu1998hz/InstructScore", cache_dir=cache_dir, model_max_length=max_src_len, use_fast=False
            )
            self.model = LlamaForCausalLM.from_pretrained("xu1998hz/InstructScore", cache_dir=cache_dir, torch_dtype=torch.bfloat16, device_map="auto")
        elif task_type == "mt_en-es":
            self.tokenizer = AutoTokenizer.from_pretrained(
                "xu1998hz/instructscore_en-es", cache_dir=cache_dir, model_max_length=max_src_len, use_fast=False
            )
            self.model = AutoModelForCausalLM.from_pretrained("xu1998hz/instructscore_en-es", cache_dir=cache_dir, torch_dtype=torch.bfloat16, device_map="auto")
        elif task_type == 'mt_en-ru':
            self.tokenizer = LlamaTokenizer.from_pretrained(
                "xu1998hz/InstructScore", cache_dir=cache_dir, model_max_length=max_src_len, use_fast=False
            )
            self.model = LlamaForCausalLM.from_pretrained("xu1998hz/instructscore_en-ru", cache_dir=cache_dir, torch_dtype=torch.bfloat16, device_map="auto").to(
                device_id
            )
        elif task_type == 'mt_en-de':
            self.tokenizer = LlamaTokenizer.from_pretrained(
                "xu1998hz/InstructScore", cache_dir=cache_dir, model_max_length=max_src_len, use_fast=False
            )
            self.model = LlamaForCausalLM.from_pretrained("xu1998hz/instructscore_en-de", cache_dir=cache_dir, torch_dtype=torch.bfloat16, device_map="auto").to(
                device_id
            )
        elif task_type == 'caption':
            self.tokenizer = LlamaTokenizer.from_pretrained(
                "xu1998hz/InstructScore", cache_dir=cache_dir, model_max_length=max_src_len, use_fast=False
            )
            self.model = LlamaForCausalLM.from_pretrained("xu1998hz/instructscore_caption", cache_dir=cache_dir, torch_dtype=torch.bfloat16, device_map="auto")
        
        elif self.task_type == 'd2t':
            self.tokenizer = LlamaTokenizer.from_pretrained(
                "xu1998hz/InstructScore", cache_dir=cache_dir, model_max_length=max_src_len, use_fast=False
            )
            self.model = LlamaForCausalLM.from_pretrained("xu1998hz/instructscore_data2text", cache_dir=cache_dir, torch_dtype=torch.bfloat16, device_map="auto")
            
        elif self.task_type == 'commonsense':
            self.tokenizer = LlamaTokenizer.from_pretrained(
                "xu1998hz/InstructScore", cache_dir=cache_dir, model_max_length=max_src_len, use_fast=False
            )
            self.model = LlamaForCausalLM.from_pretrained("xu1998hz/instructscore_commonsense", cache_dir=cache_dir, torch_dtype=torch.bfloat16, device_map="auto")

        else:
            print("Task weights are not supported!")
            exit(1)

        self.tokenizer.padding_side = "left"
        # enable batch inference by left padding
        if self.task_type != "mt_en-es":
            smart_tokenizer_and_embedding_resize(
                special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
                tokenizer=self.tokenizer,
            )
        else:
            self.tokenizer.pad_token = self.tokenizer.eos_token    
        self.model.eval()

    def score(self, ref_ls, out_ls, src_ls=None):
        assert len(ref_ls) == len(out_ls), "The number of references and outputs should be the same."
        if len(ref_ls) == 0 or len(out_ls) == 0:
            return [], []

        if isinstance(ref_ls, str):
            ref_ls = [ref_ls]
        if isinstance(out_ls, str):
            out_ls = [out_ls]
        if isinstance(src_ls, str):
            src_ls = [src_ls]
        
        if self.task_type == 'mt_zh-en':
            prompt_ls = [
                f'You are evaluating Chinese-to-English Machine translation task. The correct translation is "{ref}". The model generated translation is "{out}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don\'t lead to loss of meaning but will be noticed.'
                for ref, out in zip(ref_ls, out_ls)
            ]
        elif self.task_type == 'mt_en-de':
            prompt_ls = [
                f'You are evaluating English-to-German Machine translation task. The correct translation is "{ref}". The model generated translation is "{out}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don\'t lead to loss of meaning but will be noticed.'
                for ref, out in zip(ref_ls, out_ls)
            ]
        elif self.task_type == 'mt_en-ru':
            prompt_ls = [
                f'You are evaluating English-to-Russian Machine translation task. The correct translation is "{ref}". The model generated translation is "{out}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don\'t lead to loss of meaning but will be noticed.'
                for ref, out in zip(ref_ls, out_ls)
            ]
        elif self.task_type == 'mt_en-es':
            prompt_ls = [
                f'You are evaluating English-to-Spanish Machine translation task. The correct translation is "{ref}". The model generated translation is "{out}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don\'t lead to loss of meaning but will be noticed.'
                for ref, out in zip(ref_ls, out_ls)
            ]
        elif self.task_type == 'caption':
            prompt_ls = [
                f"""You are evaluating image captioning. The correct generation is "{ref}". The model generated translation is "{out}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."""
                for ref, out in zip(ref_ls, out_ls)
            ]
        elif self.task_type == 'd2t':
            prompt_ls = [
                f"""You are evaluating RDF-to-text task. The correct generation is "{ref}". The input of model is "{src}". The model generated output is "{out}\n". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error dimension, error type, major/minor label, error location of the model generated output and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."""
                for ref, out, src in zip(ref_ls, out_ls, src_ls)
            ]
        elif self.task_type == 'keyword-to-text':
            prompt_ls = [
                f"""You are evaluating RDF-to-text task. The correct generation is "{ref}". The input of model is "{src}". The model generated output is "{out}\n". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error dimension, error type, major/minor label, error location of the model generated output and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."""
                for ref, out, src in zip(ref_ls, out_ls, src_ls)
            ]
        elif self.task_type == 'commonsense':
            prompt_ls = [
                f"""You are evaluating commonsense text generation. The input of model is "{src}". One of the correct generations is "{ref}". The model generated output is "{out}". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated output and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."""
                for ref, out, src in zip(ref_ls, out_ls, src_ls)
            ]
        else:
            print("Other task type is not supported at moment!")
            exit(1)

        with torch.no_grad():
            batch_outputs_all = []
            scores_ls_all = []
            with tqdm(total=len(prompt_ls)) as pbar:
                for prompt_batch in batchify(prompt_ls, self.batch_size):
                    inputs = self.tokenizer(
                        prompt_batch,
                        return_tensors="pt",
                        padding=True,
                        truncation=True,
                        max_length=self.max_src_len,
                    )
                    outputs = self.model.generate(
                        inputs["input_ids"].to(self.device_id),
                        attention_mask=inputs["attention_mask"].to(self.device_id),
                        max_new_tokens=self.max_trg_len,
                        do_sample=False, 
                        temperature=0
                    )
                    batch_outputs = self.tokenizer.batch_decode(
                        outputs[:, inputs.input_ids.shape[1]:],
                        skip_special_tokens=True,
                        clean_up_tokenization_spaces=True,
                    )
                    scores_ls = [
                        (-1) * output.count("Minor\n")
                        + (-5) * output.count("Major\n")
                        for output in batch_outputs
                    ]
                    batch_outputs_all.extend(batch_outputs)
                    scores_ls_all.extend(scores_ls)
                    pbar.update(len(batch_outputs))
            return batch_outputs_all, scores_ls_all


def batchify(data: Iterable[T], batch_size: int) -> Iterable[List[T]]:
    assert batch_size > 0

    batch = []
    for item in data:
        # Yield next batch
        if len(batch) == batch_size:
            yield batch
            batch = []

        batch.append(item)

    # Yield last un-filled batch
    if len(batch) != 0:
        yield batch  


def main():
    device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")

    task_type="mt_en-es"
    refs=["Y hay una distinción muy importante allí que veremos."]
    outs=["Y hay una distinción muy anormal allí que falta veremos."]

    scorer = InstructScore(device_id=device_id, task_type=task_type, batch_size=6, cache_dir=None)
    if task_type=="commonsense" or task_type=="d2t" or task_type == "key-to-text":
        batch_outputs, scores_ls = scorer.score(ref_ls=refs, out_ls=outs, src_ls=srcs)
    else:
        batch_outputs, scores_ls = scorer.score(ref_ls=refs, out_ls=outs)
    print(batch_outputs)
    print(scores_ls)


if __name__ == "__main__":
    main()



import ast
import json
import click
import re

""" python3 code/post_process_gpt4_feedback.py -start_str "0 800 1200" -end_str "800 1200 2000" """


@click.command()
@click.option("-start_str", type=str)
@click.option("-end_str", type=str)
def main(start_str, end_str):
    non_json_err_count = 0
    miss_q7_count = 0
    q_num_err_count = 0
    cor_count = 0
    index_set = set()
    final_dict = {}

    prefix = 'The correct translation is "'
    middle_suffix = '". The model generated translation is "'
    suffix = '". Please identify all errors within each model output'

    final_str = ""
    for start_file_index in range(0, 10000, 2000):
        end_file_index = start_file_index + 2000
        out_ref_ls = open(
            f"data/wmt_samples/test_wmt20_zh-en_samples_llama_ref_data_{start_file_index}_{end_file_index}.txt",
            "r",
        ).readlines()
        for ele in out_ref_ls:
            final_str += ele

    ref_out_dict = {}
    pattern = r"Major/minor: (.*?)\n"
    for line in final_str.split("[SEP_WENDA]")[:-1]:
        cur_index, prompt_str = int(line.split("\t")[0]), line.split("\t")[1]
        if cur_index not in ref_out_dict:
            ref_out_dict[cur_index] = []
        ref = prompt_str.split(prefix)[1].split(middle_suffix)[0]
        out = prompt_str.split(middle_suffix)[1].split(suffix)[0]
        result = re.findall(pattern, prompt_str)
        result = [ele.lower() + "-error" for ele in result]
        ref_out_dict[cur_index] += [
            {"ref": ref, "out": out, "labels": result, "prompt": prompt_str}
        ]

    for start_index, end_index in zip(
        [int(ele) for ele in start_str.split()], [int(ele) for ele in end_str.split()]
    ):
        indicater_lines = open(
            f"reward_data/fixed_new_reward_samples_indicater_{start_index}_{end_index}.txt"
        ).readlines()
        final_str_ind = ""
        for ele in indicater_lines:
            final_str_ind += ele

        lines = open(
            f"reward_data/fixed_new_reward_samples_{start_index}_{end_index}.txt"
        ).readlines()
        final_str = ""
        for ele in lines:
            final_str += ele

        assert len(final_str.split("[SEP_TOK_WENDA]")) == len(
            final_str_ind.split("[SEP_TOK_WENDA]")
        )

        for ele, ele_ind in zip(
            final_str.split("[SEP_TOK_WENDA]")[:-1],
            final_str_ind.split("[SEP_TOK_WENDA]")[:-1],
        ):
            cur_ind = ele_ind.split("\t")[-1] == "True"
            cur_index, num_seq = int(ele.split("\t")[0]), int(ele.split("\t")[1])

            if cur_index not in final_dict:
                final_dict[cur_index] = {}

            if cur_ind:
                loaded_dict = {}
                loaded_dict["no_error_ind"] = cur_ind
                proc_str = ele.split("\t")[-1]
                if proc_str == "Yes":
                    loaded_dict["no_error_score"] = 0
                else:
                    loaded_dict["no_error_score"] = 1
                cor_count += 1
                loaded_dict["prompt"] = ref_out_dict[cur_index][num_seq]["prompt"]
                final_dict[cur_index][num_seq] = loaded_dict
                index_set.add(cur_index)
            else:
                proc_str = ele.split("\t")[-1].replace("\n", "").replace("  ", "")
                try:
                    loaded_dict = ast.literal_eval(proc_str)
                    # first check if Q7 is in dict
                    if "Q7" in loaded_dict:
                        q_num_check = True
                        for err in loaded_dict:
                            # check each err fields
                            if err != "Q7":
                                if len(loaded_dict[err]) != 6:
                                    q_num_check = False
                                    break
                                else:
                                    if (
                                        loaded_dict[err]["Q1"] != "A1"
                                        and len(loaded_dict[err]["Q2"]) != 2
                                        and (
                                            loaded_dict[err]["Q3"].lower() == "yes"
                                            or loaded_dict[err]["Q3"].lower() == "no"
                                        )
                                        and (
                                            loaded_dict[err]["Q4"].lower()
                                            == "major-error"
                                            or loaded_dict[err]["Q4"].lower()
                                            == "minor-error"
                                            or loaded_dict[err]["Q4"].lower()
                                            == "no-error"
                                        )
                                        and (
                                            loaded_dict[err]["Q5"].lower() == "yes"
                                            or loaded_dict[err]["Q5"].lower() == "no"
                                        )
                                        and (
                                            loaded_dict[err]["Q6"].lower() == "yes"
                                            or loaded_dict[err]["Q6"].lower() == "no"
                                        )
                                    ):
                                        q_num_check = False
                                        break
                        if q_num_check:
                            if (
                                len(ref_out_dict[cur_index][num_seq]["labels"])
                                == len(loaded_dict) - 1
                            ):
                                cor_count += 1
                                loaded_dict["ref"] = ref_out_dict[cur_index][num_seq][
                                    "ref"
                                ]
                                loaded_dict["out"] = ref_out_dict[cur_index][num_seq][
                                    "out"
                                ]
                                loaded_dict["labels"] = ref_out_dict[cur_index][
                                    num_seq
                                ]["labels"]
                                loaded_dict["prompt"] = ref_out_dict[cur_index][
                                    num_seq
                                ]["prompt"]
                                loaded_dict["no_error_ind"] = cur_ind
                                final_dict[cur_index][num_seq] = loaded_dict
                                index_set.add(cur_index)
                        else:
                            q_num_err_count += 1
                    else:
                        miss_q7_count += 1
                except Exception as e:
                    # print(e)
                    non_json_err_count += 1

    print("Corect Count: ", cor_count)
    print("Non JSON Error Count: ", non_json_err_count)
    print("Missing Q7: ", miss_q7_count)
    print("Num of missing Err Qs: ", q_num_err_count)
    print("Instance Pass Rate: ", len(index_set) / 2000)
    print(
        "Total Pass Rate (JSON): ",
        cor_count / (non_json_err_count + cor_count + miss_q7_count + q_num_err_count),
    )
    with open("reward_data/final_data_scores.json", "w") as f:
        json.dump(final_dict, f)


if __name__ == "__main__":
    main()



import glob

final_dict = {}
for file_addr in glob.glob(
    "/share/edc/home/wendaxu/reward_selections/SEScore3_output_sample_222/*_3.txt"
):
    file_name = file_addr.split("/")[-1]
    sys_name = file_name.split("test_wmt22_zh-en_")[1].split("_llama_")[0]
    final_str = ""
    lines = open(file_addr, "r").readlines()
    for ele in lines:
        final_str += ele
    if sys_name not in final_dict:
        final_dict[sys_name] = {}
    for cur_str in final_str.split("[SEP_WENDA]")[:-1]:
        sen_index, ret_index, output = (
            cur_str.split("\t")[0],
            cur_str.split("\t")[1],
            cur_str.split("\t")[2],
        )
        if sen_index not in final_dict[sys_name]:
            final_dict[sys_name][sen_index] = {}
        final_dict[sys_name][sen_index][ret_index] = output

sys_ls = [
    "LanguageX",
    "M2M100_1.2B-B4",
    "Online-A",
    "Online-B",
    "Online-G",
    "Online-W",
    "Online-Y",
    "bleurt_bestmbr",
]

output_ls = []
for sys in sys_ls:
    print(final_dict[sys].keys())
    print()
    for ele in list(final_dict[sys].values()):
        output_ls += [cur_ele for cur_ele in ele.values()]



import json
from scipy import stats
import glob

task_name = 'webnlg'

gt_ls = []
sescore3_ls = []
cor_ls = []
cov_ls = []
flu_ls = []
rel_ls = []
str_ls = []
for sys_addr in glob.glob('test_webnlg/ref/*'):
    gt_dict = json.load(open(sys_addr))
    gt_ls += [sum(ele['output'].values()) for ele in gt_dict['instances']]
    cor_ls += [ele['output']['Correctness'] for ele in gt_dict['instances']]
    cov_ls += [ele['output']['DataCoverage'] for ele in gt_dict['instances']]
    flu_ls += [ele['output']['Fluency'] for ele in gt_dict['instances']]
    rel_ls += [ele['output']['Relevance'] for ele in gt_dict['instances']]
    str_ls += [ele['output']['TextStructure'] for ele in gt_dict['instances']]

    final_str = ''
    file_name = sys_addr.split('/')[-1].replace('.json', '_0_None.txt')
    lines = open(f'test_{task_name}/output_800/{file_name}', 'r').readlines()
    for ele in lines:
        final_str += ele

    scores_ls = []
    for ele in final_str.split('[SEP_WENDA]')[:-1]:
        score = 0
        score += (-1) * ele.count('Major/minor:  Minor')
        score += (-1) * ele.count('Major/minor: Minor')
        score += (-1) * ele.count('Major/minor:  Major')
        score += (-1) * ele.count('Major/minor: Major')
        scores_ls += [score]
    sescore3_ls += scores_ls

res = stats.kendalltau(gt_ls, sescore3_ls)
print("Kendall: ", res[0])
res = stats.pearsonr(gt_ls, sescore3_ls)
print("Pearson: ", res[0])

res = stats.kendalltau(cor_ls, sescore3_ls)
print("Correctness Kendall: ", res[0])

res = stats.kendalltau(cov_ls, sescore3_ls)
print("Coverage Kendall: ", res[0])

res = stats.kendalltau(flu_ls, sescore3_ls)
print("Fluency Kendall: ", res[0])

res = stats.kendalltau(rel_ls, sescore3_ls)
print("Relvence Kendall: ", res[0])

res = stats.kendalltau(str_ls, sescore3_ls)
print("TextStructure Kendall: ", res[0])


from transformers import LlamaForCausalLM
import torch
import json
from datasets import load_dataset
from transformers import (
    AutoConfig,
    Trainer,
    TrainingArguments
)
import copy
from typing import Dict, Sequence
import transformers
from torch.utils.data import Dataset
from dataclasses import dataclass
import click


max_length = 720
padding_strategy = 'right'
DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"
IGNORE_INDEX=-100

def _tokenize_fn(strings: Sequence[str], tokenizer: transformers.PreTrainedTokenizer) -> Dict:
    """Tokenize a list of strings."""
    tokenized_list = [
        tokenizer(
            text,
            return_tensors="pt",
            padding="max_length",
            max_length=max_length,
            truncation=True,
        )
        for text in strings
    ] 
    input_ids = labels = [tokenized.input_ids[0] for tokenized in tokenized_list]
    #print(input_ids)
    input_ids_lens = labels_lens = [
        tokenized.input_ids.ne(tokenizer.pad_token_id).sum().item() for tokenized in tokenized_list
    ]
    return dict(
        input_ids=input_ids,
        labels=labels,
        input_ids_lens=input_ids_lens,
        labels_lens=labels_lens,
    )


def preprocess(sources, targets, tokenizer):
    # remove pairs where at least one record is None
    examples = [s + t for s, t in zip(sources, targets)]
    examples_tokenized, sources_tokenized = [_tokenize_fn(strings, tokenizer) for strings in (examples, sources)]
    input_ids = examples_tokenized["input_ids"]
    labels = copy.deepcopy(input_ids)
    for label, source_len in zip(labels, sources_tokenized["input_ids_lens"]):
        label[:source_len] = IGNORE_INDEX
    return dict(input_ids=input_ids, labels=labels)

class SupervisedDataset(Dataset):
    """Dataset for supervised fine-tuning."""

    def __init__(self, tokenizer: transformers.PreTrainedTokenizer):
        super(SupervisedDataset, self).__init__()
        targets = [f"{example['output']}{tokenizer.eos_token}" for example in raw_dataset]
        data_dict = preprocess(raw_dataset['input'], targets, tokenizer)
        self.input_ids = data_dict["input_ids"]
        self.labels = data_dict["labels"]

    def __len__(self):
        return len(self.input_ids)

    def __getitem__(self, i) -> Dict[str, torch.Tensor]:
        return dict(input_ids=self.input_ids[i], labels=self.labels[i])

@dataclass
class DataCollatorForSupervisedDataset(object):
    """Collate examples for supervised fine-tuning."""

    tokenizer: transformers.PreTrainedTokenizer

    def __call__(self, instances: Sequence[Dict]) -> Dict[str, torch.Tensor]:
        input_ids, labels = tuple([instance[key] for instance in instances] for key in ("input_ids", "labels"))
        input_ids = torch.nn.utils.rnn.pad_sequence(
            input_ids, batch_first=True, padding_value=self.tokenizer.pad_token_id
        )
        
        labels = torch.nn.utils.rnn.pad_sequence(labels, batch_first=True, padding_value=IGNORE_INDEX)
        return dict(
            input_ids=input_ids,
            labels=labels,
            attention_mask=input_ids.ne(self.tokenizer.pad_token_id),
        )

def make_supervised_data_module(tokenizer: transformers.PreTrainedTokenizer) -> Dict:
    """Make dataset and collator for supervised fine-tuning."""
    train_dataset = SupervisedDataset(tokenizer=tokenizer)
    data_collator = DataCollatorForSupervisedDataset(tokenizer=tokenizer)
    return dict(train_dataset=train_dataset, eval_dataset=None, data_collator=data_collator)

f = 'data/english_llama_ref_data.json'
# sanity check over the fields of json file
with open(f) as fin:
    json_data = json.load(fin)

# Load the dataset using the HuggingFace dataset library
KEY_INSTANCES = "instances"
extensions = "json"
raw_dataset = load_dataset(
    extensions,
    data_files=[f],
    field=KEY_INSTANCES,
    split="train",
    use_auth_token=None,
)

tokenizer = transformers.AutoTokenizer.from_pretrained(
        "decapoda-research/llama-7b-hf",
        model_max_length=max_length,
        padding_side=padding_strategy,
        use_fast=False,
    )

def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
    #model: transformers.PreTrainedModel,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    num_new_tokens = tokenizer.add_special_tokens(special_tokens_dict)

if tokenizer.pad_token is None:
    smart_tokenizer_and_embedding_resize(
        special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
        tokenizer=tokenizer,
    )

tokenizer.add_special_tokens(
    {
        "eos_token": DEFAULT_EOS_TOKEN,
        "bos_token": DEFAULT_BOS_TOKEN,
        "unk_token": DEFAULT_UNK_TOKEN,
    }
)

data_module = make_supervised_data_module(tokenizer=tokenizer)


import glob
import math
from scipy import stats


def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i : min(i + batch_size, len(lst))]


final_str = ""
for file_addr in sorted(glob.glob("test_commongen/output_462/*")):
    lines = open(file_addr, "r").readlines()
    final_str += "".join(lines)

lines = open("test_commongen/common_gen_test_1.txt", "r").readlines()
lines = [float(ele[:-1].split("\t")[-1]) for ele in lines]


def score_funct(ele):
    score = 0
    score += (-1) * ele.count("Major/minor:  Minor")
    score += (-5) * ele.count("Major/minor:  Major")
    return score


p, q = 0, 0
n0, n1, n2 = 0, 0, 0
h_ls, s_ls = [], []
for ele, hum in zip(
    batchify(final_str.split("[SEP_WENDA]")[:-1], 2), batchify(lines, 2)
):
    h_ls += [hum[0]]
    s_ls += [score_funct(ele[0]) - score_funct(ele[1])]
    if (hum[0] > 0 and score_funct(ele[0]) > score_funct(ele[1])) or (
        hum[0] < 0 and score_funct(ele[0]) < score_funct(ele[1])
    ):
        p += 1
    elif (
        hum[0] < 0
        and score_funct(ele[0]) > score_funct(ele[1])
        or (hum[0] > 0 and score_funct(ele[0]) < score_funct(ele[1]))
    ):
        q += 1
    elif hum[0] == 0 and score_funct(ele[0]) == score_funct(
        ele[1]
    ):  # and score_funct(ele[1]) == score_funct(ele[0])
        n0 += 1
    elif hum[0] == 0 and score_funct(ele[0]) != score_funct(ele[1]):
        n1 += 1
    elif hum[0] != 0 and score_funct(ele[0]) == score_funct(ele[1]):
        n2 += 1
    else:
        print("nothing should be here!")

print("p, q, n0, n1, n2: ", p, q, n0, n1, n2)
print("InstructScore: ", (p + n0) / (p + q + n0 + n1 + n2))



# cli.py

import click
from translation_canvas.app import create_app

@click.command()
@click.option('--host', default='127.0.0.1', help='The interface to bind to.')
@click.option('--port', default=5000, help='The port to bind to.')
@click.option('--debug', default=False, help='Enable debug mode.')
def start_server(host, port, debug):
    """Starts the Flask server."""
    create_app().run(host=host, port=port, debug=debug)

if __name__ == '__main__':
    start_server()



import os
import re
import subprocess
import psutil
from translation_canvas.readwrite_database import write_data

path_to_file = os.path.dirname(os.path.abspath(__file__))


def _delete_runs_subprocess(run_ids):
    for run_id in run_ids:
        kill_processes_by_run_id(run_id)
        print("Deleting run_id: ", run_id)
        write_data(f"DELETE FROM preds_text WHERE pred_id IN (SELECT id FROM preds WHERE run_id = {run_id}); DELETE FROM preds WHERE run_id = {run_id}; DELETE FROM runs WHERE id = {run_id}; DELETE FROM refs WHERE id NOT IN (SELECT ref_id FROM preds);")
    
def spawn_independent_process(command):
    
    process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True, start_new_session=True, universal_newlines=True)
    return process.pid
    
def split_sentence_with_queries(sentence, queries):
    """
    Split a sentence into phrases using a list of queries as delimiters.

    Args:
        sentence (str): The sentence to be split.
        queries (list): A list of queries to be used as delimiters.

    Returns:
        list: A list of phrases obtained by splitting the sentence using the queries as delimiters.
    """
    # Create a regular expression pattern to match any of the queries
    pattern = '|'.join(re.escape(query) for query in queries)
    # Split the sentence using the pattern as the delimiter
    parts = re.split(f'({pattern})', sentence, flags=re.IGNORECASE)

    # Filter out empty strings and None values
    phrases = [part.strip() for part in parts if part]

    return phrases

def read_file_content(file_path):
    with open(file_path, 'r') as file:
        return file.read(1000000)
    
def write_file_content(file_path, content):
    with open(file_path, 'w') as file:
        return file.write(content)
    
def kill_processes_by_run_id(selected_run_id):
    for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
        # print(f"Checking process {proc.info['pid']}")
        try:
            cmdline = proc.info['cmdline']
            if cmdline:
                # Convert the command line to a single string
                cmdline_str = ' '.join(cmdline)
                # Check if the command line contains the selected run_id
                if f'eval.py --run_id {selected_run_id}' in cmdline_str:
                    print(f"Killing process {proc.info['pid']} with command: {cmdline_str}")
                    proc.terminate()  # or proc.kill() if you want to force kill
        except (psutil.NoSuchProcess, psutil.AccessDenied, psutil.ZombieProcess) as e:
            print(f"Error while trying to kill process {proc.info['pid']}, error: {e}")
            pass


# pip install git+https://github.com/huggingface/transformers transformers-4.28.0.dev0

from transformers import LlamaForCausalLM
import torch
import json
from datasets import load_dataset
from transformers import Trainer, TrainingArguments
import copy
from typing import Dict, Sequence
import transformers
from torch.utils.data import Dataset
from dataclasses import dataclass
import argparse

"""deepspeed --num_gpus 8 code/finetune_llama.py"""

KEY_TYPE = "type"
KEY_INSTANCES = "instances"
IGNORE_INDEX = -100
DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"

parser = argparse.ArgumentParser(description='Arguments to InstructScore training: deepspeed --num_gpus 8 code/finetune_llama.py')
parser.add_argument('--ds_config', type=str, default="config/ds_config_zero3.json",
                    help='location of deepspeed config json')
parser.add_argument('--max_length', type=int, default=720,
                    help='max length for the input token')
parser.add_argument('--f', type=str, default="data/english_llama_ref_data.json",
                    help='file address for the training data')
parser.add_argument('--output_dir', type=str, default='weight_dir',
                    help='output location for the weight')
parser.add_argument('--padding_strategy', type=str, default="left",
                    help='padding strategy: you can use left')
parser.add_argument('--num_epoch', type=int, default=3,
                    help='Number of epoch. For example, 3')
parser.add_argument('--local_rank', type=int)
args = parser.parse_args()

do_train = True
ds_config = args.ds_config 
max_length = args.max_length
f = args.f
output_dir = args.output_dir 
padding_strategy = args.padding_strategy
num_epoch = args.num_epoch 

class SupervisedDataset(Dataset):
    """Dataset for supervised fine-tuning."""

    def __init__(self, tokenizer: transformers.PreTrainedTokenizer, raw_dataset):
        super(SupervisedDataset, self).__init__()
        targets = [
            f"{example['output']}{tokenizer.eos_token}" for example in raw_dataset
        ]
        data_dict = preprocess(raw_dataset["input"], targets, tokenizer)
        self.input_ids = data_dict["input_ids"]
        self.labels = data_dict["labels"]

    def __len__(self):
        return len(self.input_ids)

    def __getitem__(self, i) -> Dict[str, torch.Tensor]:
        return dict(input_ids=self.input_ids[i], labels=self.labels[i])


@dataclass
class DataCollatorForSupervisedDataset(object):
    """Collate examples for supervised fine-tuning."""

    tokenizer: transformers.PreTrainedTokenizer

    def __call__(self, instances: Sequence[Dict]) -> Dict[str, torch.Tensor]:
        input_ids, labels = tuple(
            [instance[key] for instance in instances] for key in ("input_ids", "labels")
        )
        input_ids = torch.nn.utils.rnn.pad_sequence(
            input_ids, batch_first=True, padding_value=self.tokenizer.pad_token_id
        )
        labels = torch.nn.utils.rnn.pad_sequence(
            labels, batch_first=True, padding_value=IGNORE_INDEX
        )
        return dict(
            input_ids=input_ids,
            labels=labels,
            attention_mask=input_ids.ne(self.tokenizer.pad_token_id),
        )


def make_supervised_data_module(tokenizer: transformers.PreTrainedTokenizer, raw_dataset) -> Dict:
    """Make dataset and collator for supervised fine-tuning."""
    train_dataset = SupervisedDataset(tokenizer=tokenizer, raw_dataset=raw_dataset)
    data_collator = DataCollatorForSupervisedDataset(tokenizer=tokenizer)
    return dict(
        train_dataset=train_dataset, eval_dataset=None, data_collator=data_collator
    )


def preprocess(sources, targets, tokenizer):
    # remove pairs where at least one record is None
    examples = [s + t for s, t in zip(sources, targets)]
    examples_tokenized, sources_tokenized = [
        _tokenize_fn(strings, tokenizer) for strings in (examples, sources)
    ]
    input_ids = examples_tokenized["input_ids"]
    labels = copy.deepcopy(input_ids)
    for label, source_len in zip(labels, sources_tokenized["input_ids_lens"]):
        label[:source_len] = IGNORE_INDEX
    return dict(input_ids=input_ids, labels=labels)


def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
    model: transformers.PreTrainedModel,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    num_new_tokens = tokenizer.add_special_tokens(special_tokens_dict)
    model.resize_token_embeddings(len(tokenizer))

    if num_new_tokens > 0:
        input_embeddings = model.get_input_embeddings().weight.data
        output_embeddings = model.get_output_embeddings().weight.data

        input_embeddings_avg = input_embeddings[:-num_new_tokens].mean(
            dim=0, keepdim=True
        )
        output_embeddings_avg = output_embeddings[:-num_new_tokens].mean(
            dim=0, keepdim=True
        )

        input_embeddings[-num_new_tokens:] = input_embeddings_avg
        output_embeddings[-num_new_tokens:] = output_embeddings_avg


def _tokenize_fn(
    strings: Sequence[str], tokenizer: transformers.PreTrainedTokenizer
) -> Dict:
    """Tokenize a list of strings."""
    tokenized_list = [
        tokenizer(
            text,
            return_tensors="pt",
            padding="max_length",
            max_length=max_length,
            truncation=True,
        )
        for text in strings
    ]
    input_ids = labels = [tokenized.input_ids[0] for tokenized in tokenized_list]
    input_ids_lens = labels_lens = [
        tokenized.input_ids.ne(tokenizer.pad_token_id).sum().item()
        for tokenized in tokenized_list
    ]
    return dict(
        input_ids=input_ids,
        labels=labels,
        input_ids_lens=input_ids_lens,
        labels_lens=labels_lens,
    )

def main():
    # sanity check over the fields of json file
    with open(f) as fin:
        json_data = json.load(fin)
        if KEY_TYPE not in json_data.keys():
            raise ValueError(
                f'"{KEY_TYPE}" field must be specified for data, e.g.'
                "{\n"
                f'   "{KEY_TYPE}: "text2text",\n'
                f'   "{KEY_INSTANCES}": [\n'
                '       { "text": "Sentence 1: This is a sentence." }\n'
                '       { "text": "Sentence 2: This is another sentence." }\n'
                f"   ]\n"
                "}"
            )

    # Load the dataset using the HuggingFace dataset library
    extensions = "json"
    raw_dataset = load_dataset(
        extensions,
        data_files=[f],
        field=KEY_INSTANCES,
        split="train",
        use_auth_token=None,
    )

    print(raw_dataset)

    tokenizer = transformers.AutoTokenizer.from_pretrained(
        "yahma/llama-7b-hf",
        model_max_length=max_length,
        padding_side=padding_strategy,
        use_fast=False,
    )

    model = LlamaForCausalLM.from_pretrained("yahma/llama-7b-hf")
    print("Loaded in model and tokenizers")
    
    if tokenizer.pad_token is None:
        smart_tokenizer_and_embedding_resize(
            special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
            tokenizer=tokenizer,
            model=model,
        )

    tokenizer.add_special_tokens(
        {
            "eos_token": DEFAULT_EOS_TOKEN,
            "bos_token": DEFAULT_BOS_TOKEN,
            "unk_token": DEFAULT_UNK_TOKEN,
        }
    )

    data_module = make_supervised_data_module(tokenizer=tokenizer, raw_dataset=raw_dataset)
    training_args = TrainingArguments(
        output_dir=output_dir,
        evaluation_strategy="no",
        per_device_train_batch_size=1,
        gradient_accumulation_steps=16,
        learning_rate=2e-5,
        weight_decay=0,
        num_train_epochs=num_epoch,
        warmup_ratio=0,
        logging_strategy="steps",
        logging_first_step=True,
        save_strategy="epoch",
        save_total_limit=3,
        seed=42,
        run_name="wandb",
        load_best_model_at_end=False,
        greater_is_better=False,
        deepspeed=ds_config,
        log_on_each_node=False,
        logging_steps=1,
        fp16=True,
        lr_scheduler_type="cosine",
    )  # tf32=True -> only for A100

    print("Start the trainer")

    if do_train:
        trainer = Trainer(
            model=model,
            args=training_args,
            train_dataset=data_module["train_dataset"],
            eval_dataset=None,
            tokenizer=tokenizer,
            data_collator=data_module["data_collator"],
            # compute_metrics=data_module.compute_metrics,
            preprocess_logits_for_metrics=None,
        )

        train_result = trainer.train()
        trainer.save_model()  # Saves the tokenizer too for easy upload
        metrics = train_result.metrics

        trainer.log_metrics("train", metrics)
        trainer.save_metrics("train", metrics)

if __name__ == "__main__":
    main()



import glob

# load in all sample data
cur_dict = {}
for file_name in glob.glob('data/wmt_samples/*'):
    lines = open(file_name, 'r').readlines()
    final_str = ''.join(lines)
    samples_ls = final_str.split('[SEP_WENDA]')[:-1]   
    for ele in samples_ls:
        if ele.split('\t')[0] not in cur_dict:
            cur_dict[ele.split('\t')[0]]=[]
        cur_dict[ele.split('\t')[0]] += [ele.split('\t')[1]]

new_dict = {}
for i in range(0, len(cur_dict), 5):
    print(cur_dict['0'][0])
    print()
    print(cur_dict['5'][0])


from mt_metrics_eval import data
import random
import json

lang = 'zh-en'
evs = data.EvalSet('wmt20', 'zh-en')

save_train_dict, save_test_dict = {"type": "text2text", "instances": []}, {"type": "text2text", "instances": []}
ref_ls = evs.sys_outputs[evs.std_ref]
for i in range(2000):
    cur_set = random.sample(list(set(evs.sys_outputs)-{evs.std_ref}), 10)
    reward_set, update_set = cur_set[:5], cur_set[5:]
    for sys in reward_set:
        cur_output = evs.sys_outputs[sys][i]
        cur_ref = ref_ls[i]
        train_prompt = f"You are evaluating Chinese-to-English Machine translation task. The correct translation is \"{cur_ref}\". The model generated translation is \"{cur_output}\". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."
        save_train_dict["instances"] += [{"input": train_prompt}]
    for sys in update_set:
        test_prompt = f"You are evaluating Chinese-to-English Machine translation task. The correct translation is \"{cur_ref}\". The model generated translation is \"{cur_output}\". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."
        save_test_dict["instances"] += [{"input": test_prompt}]

with open(f'data/english_llama_ref_reward.json', 'w') as f:
    json.dump(save_train_dict, f)

with open(f'data/english_llama_ref_test.json', 'w') as f:
    json.dump(save_test_dict, f)

print("Reward and test files are saved!")


import torch
from transformers import LlamaForCausalLM, LlamaTokenizer
from typing import Optional, Dict, Sequence
import transformers
import click
from typing import List
import time

DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"
MAX_SEQ_LEN = 720
MAX_TARGET_LENGTH = 512

ref="""According to au123.com, after a marathon debate and weeks of protests, New South Wales in Australia passed a bill to decriminalize abortion on September 26th (local time).”"""
cand=""""According to the Australian Network, after marathon debate and weeks of protests, on September 26 local time, the new state of Australia passed a bill to legalize abortion."""

txt=\
f"You are evaluating Chinese-to-English Machine translation task. The correct translation is \"{ref}\". The model generated translation is \"{cand}\". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."

def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    num_new_tokens = tokenizer.add_special_tokens(special_tokens_dict)
    tokenizer.add_special_tokens(
        {
            "eos_token": DEFAULT_EOS_TOKEN,
            "bos_token": DEFAULT_BOS_TOKEN,
            "unk_token": DEFAULT_UNK_TOKEN,
        }
    )

class LLaMA:
    def __init__(self, model, tokenizer):
        self.model = model
        self.tokenizer = tokenizer

    def generate(
        self,
        prompts: List[str],
        max_gen_len: int,
        temperature: float = 0.8,
        top_p: float = 0.95,
    ) -> List[str]:
        bsz = len(prompts)
        prompt_tokens = [self.tokenizer.encode(x) for x in prompts]

        min_prompt_size = min([len(t) for t in prompt_tokens])
        max_prompt_size = max([len(t) for t in prompt_tokens])

        total_len = min(MAX_SEQ_LEN, MAX_TARGET_LENGTH + max_prompt_size)

        tokens = torch.full((bsz, total_len), self.tokenizer.get_vocab()[DEFAULT_PAD_TOKEN]).cuda().long()
        for k, t in enumerate(prompt_tokens):
            tokens[k, : len(t)] = torch.tensor(t).long()
        input_text_mask = tokens != self.tokenizer.get_vocab()[DEFAULT_PAD_TOKEN]
        start_pos = min_prompt_size
        prev_pos = 0
        cur_status = {i: 0 for i in range(bsz)}
        for cur_pos in range(start_pos, total_len):
            if bsz == sum(cur_status.values()):
                break
            logits = self.model.forward(tokens[:, prev_pos:cur_pos]).logits[:, -1, :]
            if temperature > 0:
                probs = torch.softmax(logits / temperature, dim=-1)
                next_token = sample_top_p(probs, top_p)
            else:
                next_token = torch.argmax(logits, dim=-1)
            next_token = next_token.reshape(-1)
            # only replace token if prompt has already been generated
            next_token = torch.where(
                input_text_mask[:, cur_pos], tokens[:, cur_pos], next_token
            )
            # implement the early step criterion for generation
            if torch.sum(next_token==self.tokenizer.get_vocab()[DEFAULT_EOS_TOKEN])>0:
                for index, status in enumerate(next_token==self.tokenizer.get_vocab()[DEFAULT_EOS_TOKEN]):
                    if status:
                        cur_status[index] = 1
            tokens[:, cur_pos] = next_token

        decoded = []
        for i, t in enumerate(tokens.tolist()):
            # cut to max gen len
            t = t[: len(prompt_tokens[i]) + max_gen_len]
            # cut to eos tok if any
            try:
                t = t[: t.index(self.tokenizer.get_vocab()[DEFAULT_EOS_TOKEN])]
            except ValueError:
                pass
            decoded.append(self.tokenizer.decode(t))
        return decoded


def sample_top_p(probs, p):
    probs_sort, probs_idx = torch.sort(probs, dim=-1, descending=True)
    probs_sum = torch.cumsum(probs_sort, dim=-1)
    mask = probs_sum - probs_sort > p
    probs_sort[mask] = 0.0
    probs_sort.div_(probs_sort.sum(dim=-1, keepdim=True))
    next_token = torch.multinomial(probs_sort, num_samples=1)
    next_token = torch.gather(probs_idx, -1, next_token)
    return next_token

@click.command()
@click.option('-ckpt_addr', help="checkpoint-222", default=None)
def main(ckpt_addr):
    device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
    model = LlamaForCausalLM.from_pretrained(ckpt_addr, torch_dtype=torch.float16).to(device_id)
    model.eval()

    tokenizer = LlamaTokenizer.from_pretrained('decapoda-research/llama-7b-hf')
    smart_tokenizer_and_embedding_resize(
            special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
            tokenizer=tokenizer,
        )

    print("Vocab Size: ", len(tokenizer))

    llama = LLaMA(model, tokenizer)
    print("Loaded in model and tokenizer!")

    start_time = time.time()
    with torch.no_grad():
        decoded = llama.generate([txt]*8, max_gen_len=512, temperature=0)
        print(decoded)
    print(time.time()-start_time)

if __name__ == "__main__":
    main()


import duckdb
import os
import click

@click.command()
def setup_system():
    path_to_file = os.path.dirname(__file__)
    if not os.path.exists(os.path.join(path_to_file, "tmp")):
        os.makedirs(os.path.join(path_to_file, "tmp"))

    lock_file_path = os.path.join(path_to_file, "tmp", "duckdb.lock")
    if not os.path.exists(lock_file_path):
        open(lock_file_path, 'w').close()
        
    job_path = os.path.join(path_to_file, "jobs")
    if not os.path.exists(job_path):
        os.makedirs(job_path)
        
    uploaded_files_path = os.path.join(path_to_file, "uploaded_files")
    if not os.path.exists(uploaded_files_path):
        os.makedirs(uploaded_files_path)

    database_path = os.path.join(path_to_file, 'translation_canvas.db')
    con = duckdb.connect(database=database_path)

    results = con.execute("CREATE SEQUENCE runs_id_sequence START 1;")
    results = con.execute("CREATE SEQUENCE refs_id_sequence START 1;")
    results = con.execute("CREATE SEQUENCE src_id_sequence START 1;")
    results = con.execute("CREATE SEQUENCE preds_id_sequence START 1;")
    results = con.execute("CREATE SEQUENCE preds_text_id_sequence START 1;")

    results = con.execute("""
                        CREATE TABLE IF NOT EXISTS runs (
                            id INT PRIMARY KEY DEFAULT nextval('runs_id_sequence'), 
                            filename VARCHAR(255) NOT NULL, 
                            source_lang VARCHAR(2) NOT NULL,   
                            target_lang VARCHAR(2) NOT NULL,
                            in_progress FLOAT,
                            exit_status INT,
                            path_to_err VARCHAR(255),
                            se_score FLOAT,
                            bleu_score FLOAT,
                            comet_score FLOAT,
                            num_predictions INT,
                            run_type VARCHAR(10));""")

    results = con.execute("""CREATE TABLE IF NOT EXISTS refs (
                                id INT DEFAULT nextval('refs_id_sequence') PRIMARY KEY, 
                                source_text TEXT NOT NULL, 
                                lang VARCHAR(2));""")
    
    results = con.execute("""CREATE TABLE IF NOT EXISTS src (
                                id INT DEFAULT nextval('src_id_sequence') PRIMARY KEY, 
                                source_text TEXT NOT NULL, 
                                lang VARCHAR(2));""")

    results = con.execute("""
                        CREATE TABLE IF NOT EXISTS preds (
                            id INT DEFAULT nextval('preds_id_sequence') PRIMARY KEY,
                            source_text TEXT,
                            se_score FLOAT,
                            comet_score FLOAT,
                            num_errors INT,
                            src_id INT,
                            ref_id INT,
                            run_id INT,
                            FOREIGN KEY (src_id) REFERENCES src(id),
                            FOREIGN KEY (ref_id) REFERENCES refs(id),
                            FOREIGN KEY (run_id) REFERENCES runs(id));""")

    results = con.execute("""
                    CREATE TABLE IF NOT EXISTS preds_text (
                        id INT DEFAULT nextval('preds_text_id_sequence') PRIMARY KEY,
                            source_text TEXT NOT NULL,
                            error_type VARCHAR(255),
                            error_scale VARCHAR (8),
                            error_location TEXT,
                            error_explanation TEXT,
                            pred_id INT,
                            FOREIGN KEY (pred_id) REFERENCES preds(id));""")   

    con.close()
    


import glob
from scipy import stats

test_bagel_ls = []
test_bagel_ls_1, test_bagel_ls_2, test_bagel_ls_3 = [], [], []
index_ls = []

for i in range(8):
    file_addr = f"test_bagel/ref/bagel_gt_{i}.txt"
    lines = open(file_addr, "r").readlines()
    for line in lines:
        index = int(line[:-1].split("\t")[0])
        score_1, score_2, score_3 = (
            float(line[:-1].split("\t")[1]),
            float(line[:-1].split("\t")[2]),
            float(line[:-1].split("\t")[3]),
        )
        index_ls += [index]
        test_bagel_ls_1 += [score_1]
        test_bagel_ls_2 += [score_2]
        test_bagel_ls_3 += [score_3]
        test_bagel_ls += [(score_1 + score_2 + score_3) / 3]

final_score_ls = []
for i in range(8):
    lines = open(
        f"test_bagel/output_224/test_bagel_{i}_llama_ref_data_0_None.txt", "r"
    ).readlines()
    cur_str = "".join(lines)
    for ele in cur_str.split("[SEP_WENDA]")[:-1]:
        score = 0
        score += (-1) * ele.count("Major/minor: Minor")
        score += (-1) * ele.count("Major/minor:  Minor")
        score += (-5) * ele.count("Major/minor: Major")
        score += (-5) * ele.count("Major/minor:  Major")
        final_score_ls += [score]

index_gt_score_dict = {}
index_gt_score_dict_1 = {}
index_gt_score_dict_2 = {}
index_gt_score_dict_3 = {}
index_our_score_dict = {}
for index, gt_score, our_score in zip(index_ls, test_bagel_ls, final_score_ls):
    index_gt_score_dict[index] = gt_score
    if index not in index_our_score_dict:
        index_our_score_dict[index] = []
    index_our_score_dict[index] += [our_score]

for index, gt_score in zip(index_ls, test_bagel_ls_1):
    index_gt_score_dict_1[index] = gt_score

for index, gt_score in zip(index_ls, test_bagel_ls_2):
    index_gt_score_dict_2[index] = gt_score

for index, gt_score in zip(index_ls, test_bagel_ls_3):
    index_gt_score_dict_3[index] = gt_score

for index in index_our_score_dict:
    index_our_score_dict[index] = max(index_our_score_dict[index])

res = stats.kendalltau(
    list(index_gt_score_dict.values()), list(index_our_score_dict.values())
)
print("Kendall Total: ", res[0])

res = stats.pearsonr(
    list(index_gt_score_dict.values()), list(index_our_score_dict.values())
)
print("Pearson Total: ", res[0])

res = stats.kendalltau(
    list(index_gt_score_dict_1.values()), list(index_our_score_dict.values())
)
print("1: ", res)
res = stats.kendalltau(
    list(index_gt_score_dict_2.values()), list(index_our_score_dict.values())
)
print("2: ", res)
res = stats.kendalltau(
    list(index_gt_score_dict_3.values()), list(index_our_score_dict.values())
)
print("3: ", res)



import json
from scipy import stats

# sys_ls = ['AISP-SJTU', 'HuaweiTSC', 'JDExploreAcademy', 'Lan-Bridge', 'LanguageX', 'M2M100_1.2B-B4', 'Online-A', 'Online-B', \
# 'Online-G', 'Online-W', 'Online-Y', 'bleu_bestmbr', 'bleurt_bestmbr', 'comet_bestmbr']
from mt_metrics_eval import data

wmt = "wmt22"
src_ref = "ref"
lang = "zh-en"
if wmt == "wmt22" and src_ref == "ref" and lang == "zh-en":
    suffix = ""
else:
    suffix = "_0_None"

evs = data.EvalSet(wmt, lang)
mqm_scores = evs.Scores("seg", "mqm")
sys_ls = set(mqm_scores) - evs.human_sys_names

new_sys_ls = []
for sys in sys_ls:
    if mqm_scores[sys][0] != None:
        new_sys_ls += [sys]

print(sys_ls)
print(evs.std_ref)

sescore3_ls, gt_ls = [], []
for sys in new_sys_ls:
    ref_data = json.load(
        open(
            f"test_{wmt}_{lang}/{src_ref}/test_{wmt}_{lang}_{sys}_llama_{src_ref}_data.json"
        )
    )
    ref_ls = [ele["output"] for ele in ref_data["instances"]]
    final_str = ""
    lines = open(
        f"test_{wmt}_{lang}/sescore3_{src_ref}_output/test_{wmt}_{lang}_{sys}_llama_{src_ref}_data{suffix}.txt",
        "r",
    ).readlines()
    for ele in lines:
        final_str += ele

    print("Test sample: ", len(final_str.split("[SEP_WENDA]")[:-1]))
    scores_ls = []
    for ele in final_str.split("[SEP_WENDA]")[:-1]:
        score = 0
        score += (-1) * ele.count("Major/minor: Minor")
        score += (-5) * ele.count("Major/minor: Major")
        scores_ls += [score]

    for sescore3, gt in zip(scores_ls, ref_ls):
        # if gt < -5:
        sescore3_ls += [sescore3]
        gt_ls += [gt]

res = stats.kendalltau(sescore3_ls, gt_ls)
print(res)
res = stats.pearsonr(sescore3_ls, gt_ls)
print(res)



import click
import json
import torch
import os
from tqdm.auto import tqdm
from typing import Optional, Dict, Sequence
import transformers
from transformers import LlamaForCausalLM, LlamaTokenizer
import glob

DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"
MAX_SOURCE_LENGTH = 512
MAX_TARGET_LENGTH = 512
batch_size=1
padding_strategy = "right"
print("Max source length: ", MAX_SOURCE_LENGTH)
print("MAX target length: ", MAX_TARGET_LENGTH)
print("Batch Size: ", batch_size)
print("Padding Strategy: ", padding_strategy)

def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    num_new_tokens = tokenizer.add_special_tokens(special_tokens_dict)
    tokenizer.add_special_tokens(
        {
            "eos_token": DEFAULT_EOS_TOKEN,
            "bos_token": DEFAULT_BOS_TOKEN,
            "unk_token": DEFAULT_UNK_TOKEN,
        }
    )

def batchify(lst, batch_size):
    for i in range(0, len(lst), batch_size):
        yield lst[i:min(i+batch_size, len(lst))]

tokenizer = LlamaTokenizer.from_pretrained('decapoda-research/llama-7b-hf', model_max_length=MAX_SOURCE_LENGTH, padding_side=padding_strategy, use_fast=False)
smart_tokenizer_and_embedding_resize(
        special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
        tokenizer=tokenizer,
    )

print("Vocab Size: ", len(tokenizer))

@click.command()
@click.option('-sys_name')
@click.option('-task_name')
@click.option('-loaded', type=bool, default=False)
@click.option('-ckpt_addr', help="LLama_finetune_april_8/checkpoint-148", default=None)
@click.option('-start_index', type=int)
@click.option('-end_index', type=int)
def main(loaded, task_name, sys_name, ckpt_addr, start_index, end_index):
    if not loaded:
        if not os.path.isdir(f'test_{task_name}'):
            os.makedirs(f'test_{task_name}')
            os.makedirs(f'test_{task_name}/ref')
            index = ckpt_addr.split('-')[-1]
            print(index)
            os.makedirs(f'test_{task_name}/output_{index}')
        
        score_data = json.load(open('challenge-2020/evaluation/human-evaluation/results/en/english_humeval_data_all_teams.json'))
        index_set = set() 
        gt_dict = {}
        for data in score_data:
            index_set.add(data["sample_id"])
            if data["submission_id"] not in gt_dict:
                gt_dict[data["submission_id"]] = {}
            if data["sample_id"] not in gt_dict[data["submission_id"]]:
                gt_dict[data["submission_id"]][data["sample_id"]] = {}
            gt_dict[data["submission_id"]][data["sample_id"]]["Correctness"] = data["Correctness"]
            gt_dict[data["submission_id"]][data["sample_id"]]["DataCoverage"] = data["DataCoverage"]
            gt_dict[data["submission_id"]][data["sample_id"]]["Fluency"] = data["Fluency"]
            gt_dict[data["submission_id"]][data["sample_id"]]["Relevance"] = data["Relevance"]
            gt_dict[data["submission_id"]][data["sample_id"]]["TextStructure"] = data["TextStructure"]
        
        index_set-={'1124'}

        ref_data = json.load(open('challenge-2020/evaluation/references/references-en.json'))
        
        print("Num of sys: ", len(glob.glob('challenge-2020/submissions/rdf2text/en/*')))
        for file_name in glob.glob('challenge-2020/submissions/rdf2text/en/*'):
            cur_sys = file_name.split('/')[-1]
            final_ref_dict = {'type': "text2score", "instances": []}
            lines = open(file_name+'/primary.en', 'r').readlines()

            for index in index_set:
                ref = ref_data['entries'][int(index)-1][index]['lexicalisations'][0]['lex']
                output = lines[int(index)-1]
                ref_prompt=f"You are evaluating Chinese-to-English Machine translation task. The correct translation is \"{ref}\". The model generated translation is \"{output}\". Please identify all errors within each model output, up to a maximum of five. For each error, please give me the corresponding error type, major/minor label, error location of the model generated translation and explanation for the error. Major errors can confuse or mislead the reader due to significant change in meaning, while minor errors don't lead to loss of meaning but will be noticed."
                final_ref_dict["instances"]+=[{"input": ref_prompt, "output": gt_dict[cur_sys][index]}]
            
            with open(f'test_{task_name}/ref/test_{task_name}_{cur_sys}_llama_ref_data.json', 'w') as f:
                json.dump(final_ref_dict, f)
        print("All files are saved!")
    else:
        KEY_TYPE = "type"
        KEY_INSTANCES = "instances"
        # sanity check over the fields of json file
        with open(f'test_{task_name}/ref/test_{task_name}_{sys_name}_llama_ref_data.json') as fin:
            json_data = json.load(fin)
            if KEY_TYPE not in json_data.keys():
                raise ValueError(
                    f'"{KEY_TYPE}" field must be specified for data, e.g.'
                    '{\n'
                    f'   "{KEY_TYPE}: "text2text",\n'
                    f'   "{KEY_INSTANCES}": [\n'
                    '       { "text": "Sentence 1: This is a sentence." }\n'
                    '       { "text": "Sentence 2: This is another sentence." }\n'
                    f'   ]\n'
                    '}'
                )
        device_id = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu") 
        model = LlamaForCausalLM.from_pretrained(ckpt_addr).to(device_id)
        model.eval()
    
        print("Loaded in model and tokenizer!")
        
        index = ckpt_addr.split('-')[-1]
        save_file = open(f'test_{task_name}/output_{index}/test_{task_name}_{sys_name}_llama_ref_data_{start_index}_{end_index}.txt', 'w')

        global_step=0
        with torch.no_grad():
            with tqdm(total=len(json_data["instances"][start_index:end_index])) as pbar:
                for txts_dict in batchify(json_data["instances"][start_index:end_index], batch_size):
                    batch_txts = [txt_dict["input"] for txt_dict in txts_dict]
                    input_ids = tokenizer(batch_txts[0], return_tensors="pt").input_ids.to(device_id)
                    outputs = model.generate(input_ids, max_new_tokens=MAX_TARGET_LENGTH)
                    batch_outputs = tokenizer.batch_decode(outputs, skip_special_tokens=True, clean_up_tokenization_spaces=True)
                    save_file.write(str(global_step+start_index)+'\t'+batch_outputs[0]+'[SEP_WENDA]')
                    global_step+=1
                    pbar.update(1)

        print("File is saved!")

if __name__ == "__main__":
    main()  


import os
import argparse
from translation_canvas.readwrite_database import write_data, read_data
import subprocess
import time
import sys

path_to_file = os.path.dirname(os.path.abspath(__file__))

def spawn_eval(run_name, src_lang, tgt_lang, instructscore, bleu, comet, ref, src):
    # Define the command to run
    job_path = os.path.join(path_to_file, "jobs", run_name)   
    
    run_type = ""
    command = f"{sys.executable} {os.path.join(path_to_file, "eval.py")} --run_name {run_name} --src_lang {src_lang} --tgt_lang {tgt_lang}"
    if instructscore:
        run_type += 'i'
        command += " --instructscore True"
    if bleu:
        run_type += 'b'
        command += " --bleu True"
    if comet:
        run_type += 'c'
        command += " --comet True"
        
    if ref:
        command += " --ref True"
    if src:
        command += " --src True"
    
    write_data(f"INSERT INTO runs (filename, source_lang, target_lang, in_progress, run_type) VALUES ('{run_name}', '{src_lang}', '{tgt_lang}', 0, '{run_type}');")
    run_id = read_data("SELECT id FROM runs ORDER BY id DESC LIMIT 1")[0][0]
    command += f" --run_id {run_id}"
    
    
    # Execute the command
    print(f"executing command: {command}")
    process = subprocess.run(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
    # Print the stderr from the subprocess
    if process.stderr:
        print(f"stderr: {process.stderr.decode()}")
    # Retrieve the process ID
    status = process.returncode
    print(f"Process {run_name} finished with status {status}")
    # Update the database with the process ID
    write_data(f"UPDATE runs SET exit_status = {status} WHERE id = '{run_id}'")
    # if status == 0:
    #     os.remove(os.path.join(job_path, f"{run_name}_err.txt"))
    #     os.remove(os.path.join(job_path, f"{run_name}_out.txt"))
    #     os.remove(os.path.join(job_path, f"{run_name}_extracted.json"))
    #     os.rmdir(job_path)
    # else:
    #     write_data(f"UPDATE runs SET path_to_err = '{os.path.join(job_path, f"{run_name}_err.txt")}' WHERE id = '{run_id}'")


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--run_name', type=str, required=True)
    parser.add_argument('--src_lang', type=str, required=True)
    parser.add_argument('--tgt_lang', type=str, required=True)
    parser.add_argument('--instructscore', type=bool, default=False)
    parser.add_argument('--bleu', type=bool, default=False)
    parser.add_argument('--comet', type=bool, default=False)
    parser.add_argument('--ref', type=bool, default=False)
    parser.add_argument('--src', type=bool, default=False)
    args = parser.parse_args()
    
    print(f"Spawning evaluation for {args.run_name} with src_lang={args.src_lang}, tgt_lang={args.tgt_lang}, instructscore={args.instructscore}, bleu={args.bleu}, comet={args.comet} and ref={args.ref}, src={args.src}")

    spawn_eval(args.run_name, args.src_lang, args.tgt_lang, args.instructscore, args.bleu, args.comet, args.ref, args.src)


import duckdb
import os
import fasteners
from datetime import datetime
import time
import random

path_to_db = os.path.join(os.path.dirname(__file__), 'translation_canvas.db') 
path_to_rwlock = os.path.join(os.path.dirname(__file__), 'tmp', 'duckdb.lock')
if not os.path.exists(path_to_rwlock):
    open(path_to_rwlock, 'w').close()
pid = os.getpid()



def write_data(query: str, logging = False) -> list:
    rw_lock = fasteners.InterProcessReaderWriterLock(path_to_rwlock)
    timer = 1
    success = False
    while not success:
        try:
            with rw_lock.write_lock():
                if logging:
                    log("writing data lock acquired")
                conn = duckdb.connect(path_to_db)
                results = conn.execute(query).fetchall()
                conn.commit()
                conn.close()
                success = True
            print("success")
        except duckdb.IOException:
            timer *= random.uniform(1, 2)           # exponential backoff
            print(f"backing off for {timer} seconds")
            time.sleep(timer)

    if logging:
        log("writing data lock released")
        
    return results

def read_data(query: str, logging=False) -> list:
    rw_lock = fasteners.InterProcessReaderWriterLock(path_to_rwlock)
    timer = 1
    success = False
    while not success:
        try:
            with rw_lock.read_lock():
                if logging:
                    log("reading data lock acquired")
                conn = duckdb.connect(path_to_db, read_only=True)
                result = conn.execute(query).fetchall()
                conn.close()
                success = True
        except duckdb.IOException:
            timer *= random.uniform(1, 2)       # exponential backoff
            print(f"backing off for {timer} seconds")
            time.sleep(timer)
        
    if logging:
        log("reading data lock released")
        
    return result

def read_data_df(query: str, logging=False) -> list:
    rw_lock = fasteners.InterProcessReaderWriterLock(path_to_rwlock)
    timer = 1
    success = False
    while not success:
        try:
            with rw_lock.read_lock():
                if logging:
                    log("reading data lock acquired for df")
                conn = duckdb.connect(path_to_db, read_only=True)
                result = conn.query(query).df()
                conn.close()
                success = True
        except duckdb.IOException:
            timer *= random.uniform(1, 2)       # exponential backoff
            print(f"backing off for {timer} seconds")
            time.sleep(timer)
        
    if logging:
        log("reading data lock released for df")
        
    return result

def log(message: str):
    with open(os.path.join(os.path.dirname(__file__), 'tmp', 'log.txt'), 'a') as log_file:
        now = datetime.now()
        dt_string = now.strftime("%d/%m/%Y %H:%M:%S")
        log_file.write(f"{dt_string}: {pid}: {message}\n")
        log_file.flush()
        os.fsync(log_file)



import os
import subprocess
import sys
import json
from translation_canvas.utils import read_file_content, write_file_content, spawn_independent_process, _delete_runs_subprocess
from translation_canvas.readwrite_database import read_data, read_data_df
from translation_canvas.setup import setup_system

from flask import Flask, render_template, request, session, jsonify, send_from_directory, redirect, url_for
import secrets
import rocher.flask
from werkzeug.utils import secure_filename

import plotly, plotly.express as px
import plotly.graph_objs as go
import plotly.utils
import pandas as pd
from datetime import datetime
import multiprocessing
from dotenv import load_dotenv

path_to_file = os.path.dirname(os.path.abspath(__file__))
sys.path.append(path_to_file)
# Define the number of items per page
ITEMS_PER_PAGE = 10
help_text_json = json.load(open(os.path.join(path_to_file, 'help_text.json')))
UPLOAD_FOLDER = os.path.join(path_to_file, "uploaded_files")  # Specify your upload folder path
SCRIPTS_BASEPATH = os.path.join(path_to_file, "file_extraction_scripts")  # Specify your flaskcode resource basepath
CUDA_DEVICES_FILE = os.path.join(path_to_file, "tmp", "cuda_devices.json")
logging = False
run_id_dict = {}
ranking_log = os.path.join(path_to_file, "tmp", "ranking.log")
if not os.path.exists(ranking_log):
    open(ranking_log, 'w').close()


def create_app(test_config=None):
    """
    Create and configure the Flask app.

    Args:
        test_config (dict, optional): Configuration dictionary for testing. Defaults to None.

    Returns:
        Flask: The configured Flask app.
    """
    app = Flask(__name__)
    app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER
    app.config['SECRET_KEY'] = secrets.token_hex(16)
    rocher.flask.editor_register(app)
    
    #create run_id_dict
    results = read_data("SELECT id, filename FROM runs;")
    run_id_dict = {result[0]: result[1] for result in results}

    @app.route('/favicon.ico')
    def favicon():
        return send_from_directory(os.path.join(app.root_path, 'static'),
                               'favicon.ico', mimetype='image/vnd.microsoft.icon')
    
    @app.route('/')
    def index():
        """
        Render the index.j2 template.

        Returns:
            str: The rendered HTML template.
        """
        help_text = help_text_json["index"]
        if not os.path.exists(os.path.join(path_to_file, ".env")) or not os.path.exists(os.path.join(path_to_file, "translation_canvas.db")):
            setup_system()
            return render_template('setup.j2',  title='InstructScore Visualizer', help_text=help_text)
        
        
        # clean out session
        session.clear()
        runs = read_data("SELECT id, filename, source_lang, target_lang, in_progress, se_score, bleu_score, num_predictions, run_type, exit_status FROM runs ORDER BY target_lang, se_score, bleu_score DESC;", logging=logging)
        print(runs)
        table_data = []
        for run in runs:
            se_score = round(float(run[5]), 3) if run[5] else 'N/A'
            bleu_score = round(float(run[6]), 3) if run[6] is not None else 'N/A'
            status = None
            run_type = run[8] if run[8] else 'no eval'
            
            if run[4] and run[4] > 0:
                if run[4] >= 1:
                    status = 'Completed'
                else:
                    status = f'{round(run[4]*100, 2)}%'
            else:
                status = 'Starting'
            
            if run[9] is not None and run[9] != 0:
                status = f"Error with status {run[9]}"
                    
            table_data.append({'id': run[0], 'filename': run[1], 'source_lang': run[2], 'target_lang': run[3], 'status': status, 'se_score': se_score, 'bleu_score': bleu_score, 'num_predictions': run[7], 'run_type': run_type})
                
        return render_template('index.j2', help_text=help_text, table_data=table_data)
    
    @app.route('/setup', methods=['GET', 'POST'])
    def setup():
        """
        Setup the environment for the app.

        Returns:
            str: The rendered HTML template.
        """
        if request.method == 'GET':
            help_text = help_text_json["get_system_info"]
            
            return render_template('get_system_info.j2', help_text=help_text)
        if request.method == 'POST':
            data = request.form
            gpu_ids = data.get('gpu_ids', None)
            cache_dir = data.get('cache_dir', None)
            file = open(os.path.join(path_to_file, ".env"), 'w')
            if gpu_ids:
                file.write(f"AVAILABLE_GPU_IDS={gpu_ids}\n")
            file.write(f"CACHE_DIR='{cache_dir}'\n")
            file.close()
            
            help_text = help_text_json["index"]
            return redirect(url_for('index'))
    
    @app.route('/process', methods=['POST'])
    def process_input_form():
        """
        Process the log input from the user.

        Returns:
            str: The rendered HTML template.
        """
        print(f"session in process_input_form: {session}")
        print(f"process_input_form request.form: {request.form}")
        tgt = session['step1_data']['target_lang']
        src = session['step1_data']['source_lang']
        evaluation_type = session['step1_data']['evaluation_type']
        data_types = session['step1_data']['data_types']
        memorable_name = session['step1_data']['memorable_name']
        new_file = os.path.join(path_to_file, 'jobs', memorable_name, f'{memorable_name}_extracted.json')
        command = f"{sys.executable} {os.path.join(path_to_file, 'spawn_eval_and_monitor.py')} --run_name {memorable_name} --src_lang {src} --tgt_lang {tgt}"
        if session['step1_data']['input_method'] != 'file':
            data_dict = request.form.to_dict()
            extracted_data = []
            for i in range(len(data_dict)):
                if data_dict.get(f'prediction{i}', -1) != -1:
                    extracted_data.append({'prediction': data_dict[f'prediction{i}']})
                if data_dict.get(f'reference{i}', -1) != -1:
                    extracted_data[i]['reference'] = data_dict[f'reference{i}']
                if data_dict.get(f'source{i}', -1) != -1:
                    extracted_data[i]['source'] = data_dict[f'source{i}']
            
            if not os.path.exists(os.path.join(path_to_file, 'jobs', memorable_name)):
                os.mkdir(os.path.join(path_to_file, 'jobs', memorable_name))
                
            json.dump(extracted_data, open(new_file, 'w'), indent=4)
    
        if evaluation_type:
            if 'instructscore' in evaluation_type:
                command += " --instructscore True"
            if 'bleu' in evaluation_type:
                command += " --bleu True"  
            if 'comet' in evaluation_type:
                command += " --comet True"
        
        if data_types == 'ref':
            command += " --ref True"
        elif data_types == 'src':
            command += " --src True"
        else:
            command += " --ref True --src True"
        print(f"command: {command}")      
        spawn_independent_process(command)
        help_text = help_text_json["process_input_form"]
        return render_template('log_output.j2', memorable_name=memorable_name, file_name=new_file, help_text=help_text)
    
    @app.route('/input_form/step1', methods=['GET', 'POST'])
    def basic_info():
        """
        Render the basic_info.j2 template.

        Returns:
            str: The rendered HTML template.
        """
        help_text = help_text_json["basic_info"]
        source_languages = ['zh', 'en']
        target_languages = {
            'zh': ['en'],
            'en': ['es', 'ru', 'de']
        }
        language_names = {
        'zh': 'Chinese',
        'en': 'English',
        'es': 'Spanish',
        'ru': 'Russian',
        'de': 'German'
        }
        
        load_dotenv()
        available_gpus = os.getenv('AVAILABLE_GPU_IDS')
        available_gpus = available_gpus.split(',') if available_gpus else []
        print(f"available_gpus: {available_gpus}")
        
        return render_template('input_form.j2', help_text=help_text, source_languages=source_languages, 
                               target_languages=target_languages, 
                               language_names=language_names, available_gpus=available_gpus)
    
    @app.route('/input_form/step2', methods=['GET', 'POST'])
    def file_input():
        """
        Render the file_input.j2 template.

        Returns:
            str: The rendered HTML template.
        """
        help_text = help_text_json["file_input"]
        
        if 'gpus' in request.form:
            json.dump(request.form.getlist('gpus'), open(CUDA_DEVICES_FILE, 'w'), indent=4)
        

        
        print(f"request.form: {request.form}, session: {session}")
        if 'file_options' in request.form:              # user has chosen a file type
            
            if 'file_data' in request.form:
                filename = request.form[1]['file']
                file_type = request.form[1]['file_options']    
                
            elif 'file_upload' in request.files and request.files['file_upload'].filename != '':
                file = request.files['file_upload']
                filename = os.path.join(app.config['UPLOAD_FOLDER'], secure_filename(file.filename))
                file.save(filename)
                file_type = request.form['file_options']
                
            else:
                filename = request.form['file']
                if not os.path.exists(filename):
                    render_template("error.j2", error_message="File not found")
                file_type = request.form['file_options']
            
            memorable_name = session['step1_data']['memorable_name']
            session['step1_data']['file'] = filename
            source_code = read_file_content(os.path.join(path_to_file, 'file_extraction_scripts', f'extract_pairs_from_{file_type}.py'))
            file_content = read_file_content(filename)
            return render_template('editor.j2', source_code=source_code, file_content=file_content, file_type=file_type, memorable_name=memorable_name, filename=filename, help_text=help_text)
        
        if 'input_method' in request.form or session['step1_data']:
            session['step1_data'] = request.form.to_dict()
            evaluation_type = session['step1_data']['evaluation_type'] if 'step1_data' in session and 'evaluation_type' in session['step1_data'] else None
            if 'evaluation_type' in request.form:
                evaluation_type = request.form.getlist('evaluation_type')
            session['step1_data']['evaluation_type'] = evaluation_type
            print(f"current session in file_input right after input_form: {session}")
            
            data_types = session['step1_data']['data_types'] if 'step1_data' in session and 'data_types' in session['step1_data'] else None
            if 'data_types' in request.form:
                data_types = request.form['data_types']
                
            src = False if data_types == 'ref' else True
            ref = False if data_types == 'src' else True
            print(f"src: {src}, ref: {ref}")
                
                
            if request.form['input_method'] == 'file':
                file_options = {
                    "JSON (.json)": "json",
                    "SimulEval output (.log)": "log",
                    "CSV (.csv)": "csv",
                    "TSV (.tsv)": "tsv",
                    "XML (.xml)": "xml",
                    "Text (.txt) ": "txt",
                    }
                return render_template('file_input.j2', help_text=help_text, file_options=file_options) 
            else:
            
                return render_template('manual_input.j2', help_text=help_text, src=src, ref=ref)
    
    @app.route('/input_form/step3', methods=['POST'])
    def preview_pairs():
        """
        Preview the extracted pairs from the log file.

        Returns:
            str: The rendered HTML template.
        """
        
        source_code = request.form['source_code']
        file_type = request.form['file_options']
        filename = request.form['file']
        memorable_name = request.form['memorable_name']
        session['step1_data']['file'] = filename
        print(f"session in preview_pairs: {session}")
        write_file_content(os.path.join(path_to_file, 'file_extraction_scripts', f'extract_pairs_from_{file_type}.py'), source_code)
        results = subprocess.run([sys.executable, os.path.join(path_to_file, 'file_to_json.py'), 
                        '--file_name', filename, '--memorable_name', memorable_name, '--file_type', file_type],
                        cwd=path_to_file,
                        capture_output=True,
                        text=True)
        
        if os.path.exists(os.path.join(path_to_file, 'jobs', memorable_name, f'{memorable_name}_extracted.json')):
            pairs = read_file_content(os.path.join(path_to_file, 'jobs', memorable_name, f'{memorable_name}_extracted.json'))
            file_content = read_file_content(filename)
            help_text = help_text_json["preview_pairs_success"]
            
            return render_template('preview_pairs_success.j2', pairs=pairs, source_code = source_code, file_type=file_type, filename=filename, memorable_name=memorable_name, file_content=file_content, help_text=help_text, err=results.stderr, out = results.stdout)
        else:
            file_content = read_file_content(filename)
            help_text = help_text_json["preview_pairs_failure"]
            return render_template('preview_pairs_failure.j2', source_code=source_code, file_type=file_type, filename=filename, memorable_name=memorable_name, file_content=file_content, err=results.stderr, out = results.stdout, help_text=help_text)
    
    
    @app.route('/instruct_in', methods=['GET'])
    def instruct_in():
        """
        Render the instruct_in.j2 template.

        Returns:
            str: The rendered HTML template.
        """
        results = read_data("SELECT id, filename FROM runs WHERE in_progress > 0 ORDER BY se_score DESC;", logging=logging)
        options = {result[0]: result[1] for result in results } 
        help_text = help_text_json["instruct_in"]
        return render_template('instruct_in.j2', help_text=help_text, options=options)
    
    
    @app.route('/dashboard', methods=['GET', 'POST'])
    def dashboard():
        """
        Render the dashboard.j2 template.

        Returns:
            str: The rendered HTML template.
        """
        help_text = help_text_json["dashboard"]
        # collect counts of types of errors for each run
        if 'ids' in request.form:
            ids = request.form.get('ids', '[]')
            ids = tuple(json.loads(ids))
        elif 'files' in request.form:
            ids = tuple(request.form.getlist('files'))
        
        stats_dict = {}
        run_types = read_data(f"SELECT run_type FROM runs WHERE id IN {ids}", logging=logging)
        instructscore = False
        comet = False
        bleu = False
        for id, run_type in zip(ids, run_types):
            if 'i' in run_type[0]:
                if 'instruct' not in stats_dict:
                    stats_dict['instruct'] = []
                    instructscore = True
                stats_dict['instruct'].append(id)
            if 'c' in run_type[0]:
                if 'comet' not in stats_dict:
                    stats_dict['comet'] = []
                    comet = True
                stats_dict['comet'].append(id)
            if 'b' in run_type[0]:
                bleu = True
        
        categories = []

        instructscore_distribution_graph = None
        error_type_graph = None
        if 'instruct' in stats_dict:
            stats_dict['instruct'] = tuple(stats_dict['instruct'])
            categories.append('InstructScore')
            
            query = f"""
                WITH RankedErrors AS (
                    SELECT preds.run_id, 
                        error_type, 
                        COUNT(*) AS Count,
                        ROW_NUMBER() OVER (PARTITION BY preds.run_id ORDER BY COUNT(*) DESC) AS row_num
                    FROM preds_text
                    JOIN preds ON preds_text.pred_id = preds.id
                    WHERE error_type IS NOT NULL
                    AND preds.run_id IN {stats_dict['instruct']}
                    GROUP BY preds.run_id, error_type
                )
                SELECT run_id, error_type AS 'Error Type', Count
                FROM RankedErrors
                WHERE row_num <= 10
                ORDER BY Count, run_id DESC;
                """
            error_type_df = read_data_df(query, logging=logging)
            error_type_df['Run'] = error_type_df['run_id'].apply(get_filename)  # Add the run filename directly to the DataFrame

            query = construct_distribution_query('se_score', 'InstructScore', stats_dict['instruct'], precision=0)
            instructscore_distribution_df = read_data_df(query, logging=logging)
            instructscore_distribution_df['Run'] = instructscore_distribution_df['run_id'].apply(get_filename)
            
            error_type_graph = create_histogram(error_type_df, xaxis_title='Error Type', yaxis_title='Count', title='InstructScore: Frequency of types of errors')
            instructscore_distribution_graph = create_histogram(instructscore_distribution_df, xaxis_title='InstructScore', yaxis_title='Count', title='InstructScore: Distribution of scores per instance')
                
                
        comet_distribution_graph = None
        if comet:
            stats_dict['comet'] = tuple(stats_dict['comet'])
            categories.append('COMET')
            
            query = construct_distribution_query('comet_score', 'CometScore', stats_dict['comet'])
            comet_distribution_df = read_data_df(query, logging=logging)
            comet_distribution_df['Run'] = comet_distribution_df['run_id'].apply(get_filename)
            # Calculate the bar width
            comet_distribution_graph = create_histogram(comet_distribution_df, xaxis_title='CometScore', yaxis_title='Count', title='COMET: Distribution of scores per instance', bar_width=None)
                
        if bleu:
            categories.append('BLEU')
            

        query = f"""
                SELECT id AS run_id, 
                    COALESCE(se_score, NULL, 0) AS InstructScore, 
                    COALESCE(bleu_score, NULL, 0) AS BLEU, 
                    COALESCE(comet_score, NULL, 0) AS COMET 
                    FROM runs 
                    WHERE id IN {ids};"""
        scores_df = read_data_df(query, logging=logging)
        # Normalize columns between 0.25 and 0.9
        
        ranges = {
            "InstructScore": [-20, -6], # InstructScore
            "BLEU": [0, 45], # BLEU
            "COMET": [0, 0.8] # COMET
        }
        normalized_df = pd.DataFrame()
        for i, category in enumerate(categories):
            normalized_df[category] = scores_df[category].apply(lambda x: (x - ranges[category][0]) / (ranges[category][1] - ranges[category][0]))
        normalized_df['run_id'] = scores_df['run_id']
        scores_radar_chart = create_radar_chart(scores_df, normalized_df, categories)
        
        session['compare_systems_ids'] = ids
       
        return render_template('dashboard.j2', help_text=help_text, instructscore=instructscore, bleu=bleu, comet=comet, error_type_graph=error_type_graph, instructscore_distribution_graph=instructscore_distribution_graph, comet_distribution_graph=comet_distribution_graph, scores_radar_chart=scores_radar_chart)

    
    def create_radar_chart(df, normalized_df, categories):

        # Define colors for the radar chart
        
        contrast_colors = ['rgba(63, 81, 181, 0.3)', 'rgba(233, 30, 99, 0.3)', 'rgba(255, 152, 0, 0.3)', 'rgba(76, 175, 80, 0.3)', 'rgba(0, 188, 212, 0.3)', 'rgba(156, 39, 176, 0.3)', 'rgba(255, 235, 59, 0.3)']

        # Create radar chart
        fig = go.Figure()
        
        # Add traces for each run_id
        for index, row in normalized_df.iterrows():
            text = [f'{category}: {round(df[category][index], 2)}' for category in categories]
            text.append(f'{categories[0]}: {round(df[categories[0]][index], 2)}')  # close the loop
            fig.add_trace(go.Scatterpolar(
                r=[row[category] for category in categories] + [row[categories[0]]],  # close the loop
                theta=categories + [categories[0]],  # close the loop
                fill='toself',
                fillcolor=contrast_colors[index % len(contrast_colors)],
                line_color=contrast_colors[index % len(contrast_colors)],
                name=f'{get_filename(row["run_id"])}',
                text=text,
                textposition='top center',
                marker=dict(size=10)  # Adjust marker size here
            ))

        # Customize layout
        fig.update_layout(
            polar=dict(
                bgcolor='white',
                radialaxis=dict(
                    visible=False,
                    range=[0, 1]
                ),
                angularaxis=dict(
                    tickvals=[i for i in range(len(categories))],  # Add ticks for the categories
                    ticktext=categories,
                    linecolor='black',
                    linewidth=2
                )
            ),
            showlegend=True,
            plot_bgcolor='white',  # Set the plot background to white
            paper_bgcolor='white',  # Set the paper background to white
            title="Scores: "+', '.join([category for category in categories]),
            title_font=dict(color='black', size=24),
            legend_title_text='Runs'
        )

        radar_chart = json.dumps(fig, cls=plotly.utils.PlotlyJSONEncoder)
        return radar_chart
    
    def create_histogram(df, xaxis_title, yaxis_title, title, bar_width=None):
        # Create the bar chart using Plotly
        fig = px.bar(
            df,
            x=xaxis_title,
            y=yaxis_title,
            color='Run',
            barmode='group',
            title=title,
            labels={'Category': 'Category', 'Count': 'Count'}
        )
        

        
        # Update the layout with specified colors
        fig.update_layout(
            plot_bgcolor='white',
            font=dict(color='black'),
            title_font=dict(color='black', size=24),
            xaxis=dict(
                showgrid=False, 
                linecolor='black', 
                ticks='',  # Hide tick marks
                tickvals = [],  # Hide tick marks
                ticktext=[],  # Hide tick text
                automargin=True
            ),
            yaxis=dict(
                showgrid=False, 
                gridcolor='#f4f4f9', 
                linecolor='black', 
                ticks='outside',
                automargin=True
            ),
            legend_title_text='Runs',
        )
        if bar_width:
            fig.update_traces(width=bar_width)

        
        # Convert the plot to JSON
        histogram = json.dumps(fig, cls=plotly.utils.PlotlyJSONEncoder)
        return histogram
    
    def construct_distribution_query(score, score_name, ids, bins=10.0, precision=2):
        return f"""
            WITH MinMax AS (
                SELECT 
                    run_id,
                    MIN({score}) AS min_score,
                    MAX({score}) AS max_score
                FROM preds 
                WHERE preds.run_id IN {ids}
                GROUP BY run_id
            ),
            BinInfo AS (
                SELECT 
                    run_id,
                    min_score,
                    max_score,
                    (max_score - min_score) / {bins} AS bin_size
                FROM MinMax
            ),
            BinnedScores AS (
                SELECT 
                    preds.run_id,
                    {score},
                    min_score,
                    bin_size,
                    FLOOR(({score} - min_score) / bin_size) AS bin_index
                FROM preds
                JOIN BinInfo ON preds.run_id = BinInfo.run_id
                WHERE preds.run_id IN {ids}
            ),
            RangeCounts AS (
                SELECT 
                    run_id,
                    bin_index,
                    MIN(min_score + bin_index * bin_size) AS range_start,
                    MAX(min_score + (bin_index + 1) * bin_size - 1) AS range_end,
                    COUNT(*) AS count
                FROM BinnedScores
                GROUP BY run_id, bin_index
            )
            SELECT 
                run_id,
                ROUND((range_start + range_end) / 2, {precision}) AS {score_name},
                count AS Count
            FROM RangeCounts
            ORDER BY run_id, range_start;
        """
    
    @app.route('/visualize_instruct', methods=['POST','GET'])
    def visualize_instruct():
        """
        Visualize the instruct score.

        Returns:
            str: The rendered HTML template.
        """
        # Get page number from the request, default to 1 if not provided
        print(f'request.form in visualize_instruct: {request.form}')
        if 'ids' in request.form:
            ids = request.form.get('ids', '[]')
            files = tuple(json.loads(ids))
        
        elif 'files' in request.form:
            files = request.form.getlist('files')
            print(f"files fom next page: {files}, type: {type(files)}")
            # files = json.loads(files[0])
        
        elif 'compare_systems_ids' in session:
            files = session['compare_systems_ids']
            print(f"files from session: {files}")
        
        else:
            files = request.form.getlist('selected_options')
        
        if isinstance(files, str):
            files = [files]
        
        search_options = []
        search_texts = []
        search_query = None
        conjunctions = []
        if 'search_options[]' and 'search_texts[]' in request.form:
            search_options = request.form.getlist('search_options[]')
            search_texts = request.form.getlist('search_texts[]')
            search_texts = [text.replace("'", "''") for text in search_texts]
            conjunctions = request.form.getlist('conjunctions[]')
            if len(search_options) > 0 and len(search_texts) > 0:
                search_query = construct_full_query(search_options, search_texts, conjunctions)
            print(f"search_option: {search_options}")
            print(f"search_text: {search_texts}")
            print(f"conjunctions: {conjunctions}")
        
        if 'search_query' in session and session['search_query'] and not search_query:
            search_query = session['search_query']
            search_options = session['search_options']
            search_texts = session['search_texts']
            conjunctions = session['conjunctions']
        session['search_query'] = search_query
        session['search_options'] = search_options
        session['search_texts'] = search_texts
        session['conjunctions'] = conjunctions
        
        
        print(f"session is {session}")
        input_data = []
        print(f"Selected files: {files}")
        
                    
        page_number = int(request.form.get('current_page', 1))
        print(f"Page number: {page_number}")
        
        load_items_per_page = ITEMS_PER_PAGE//len(files)
        load_items_per_page = load_items_per_page if load_items_per_page >= 1 else 1
        
        # Calculate the starting index based on the page number
        start_index = (page_number - 1) * load_items_per_page
        
        # get ids for files selected
        
        run_ids = tuple(files)
        
        # first get all references used in the selected files
        print(f"search_query: {search_query}")
        
        
        read_query = (
            f"""SELECT DISTINCT ref_id,
                src_id,    
                s.source_text,
                r.source_text
                FROM 
                    preds p
                LEFT JOIN 
                    src s ON p.src_id = s.id
                LEFT JOIN 
                    refs r ON p.ref_id = r.id
                WHERE 
                    run_id IN {run_ids} 
               """
                            )
        if search_query:
            read_query += f"AND p.id IN ({search_query}) "
        
        read_query += f"""
             ORDER BY
                    ref_id, src_id
                OFFSET 
                        {start_index} 
                LIMIT 
                    {load_items_per_page};
            """
        total_items_query = (
                              f"SELECT COUNT( * ) FROM (SELECT DISTINCT ref_id, src_id FROM preds WHERE run_id IN {run_ids})  "
                            )
        if search_query:
            total_items_query = f"SELECT COUNT( * ) FROM (SELECT DISTINCT ref_id, src_id FROM preds WHERE run_id IN {run_ids} AND preds.id IN ({search_query}));"
        print(search_query)
        total_items = read_data(total_items_query, logging=logging)[0][0]
        print(total_items)
        
        results = read_data(read_query, logging=logging)
        ref_ids = tuple([result[0] for result in results])
        src_ids = tuple([result[1] for result in results])
        
        ref_ids_sql = f"({','.join([str(ref_id) for ref_id in ref_ids if ref_id])})"
        src_ids_sql = f"({','.join([str(src_id) for src_id in src_ids if src_id])})"
        input_data = {(result[0], result[1]): {'reference': result[3], 'runs' : {}, "source": result[2]} for result in results}       # use ref_id and src_id as a unique id for each instance
        print(f"ref_ids: {ref_ids}")
        print(f"src_ids: {src_ids}")
        print(f"run_ids: {run_ids}")
        read_query = (
                f"""SELECT preds_text.source_text, 
                            preds_text.error_type, 
                            preds_text.error_scale, 
                            preds_text.error_explanation, 
                            runs.filename, 
                            preds.ref_id, 
                            preds.id, 
                            preds.se_score, 
                            preds.comet_score,
                            preds.src_id,
                            preds_text.id
                            FROM preds 
                            JOIN preds_text 
                            ON (preds_text.pred_id = preds.id) 
                            JOIN runs
                            ON (runs.id = preds.run_id)
                            WHERE run_id IN {run_ids} 
                            """
                )
        
        if len(ref_ids_sql) > 2 and len(src_ids_sql) > 2:
            read_query += f"AND (preds.ref_id IN {ref_ids_sql} OR preds.src_id IN {src_ids_sql}) "
        elif len(ref_ids_sql) > 2:
            read_query += f"AND preds.ref_id IN {ref_ids_sql} "
        elif len(src_ids_sql) > 2:
            read_query += f"AND preds.src_id IN {src_ids_sql} "
        else:
            read_query += "AND 1=0 "
        read_query += f"ORDER BY preds.ref_id, preds.src_id;"
        print(f"read_query: {read_query}")
        results = read_data(read_query, logging=logging)
        
        does_search_have_error = False
        if len(results) > 0:
            for option in search_options:
                if 'error' in option and not does_search_have_error:
                    pred_ids = tuple([result[6] for result in results])
                    preds_text_seach_query = construct_pred_text_query(search_options, search_texts, conjunctions, pred_ids)
                    print(f"this is preds_text_seach_query: {preds_text_seach_query}")
                    preds_text_search_results = read_data(preds_text_seach_query, logging=logging)
                    print(f"preds_text_search_results: {preds_text_search_results}")
                    pred_text_search_ids = tuple([pred_text[0] for pred_text in preds_text_search_results])
                    does_search_have_error = True
            
        for result in results:          
            cur_filename = result[4]
            key = (result[5], result[9])
            if key in input_data:
                if cur_filename not in input_data[key]['runs']:
                    input_data[key]['runs'][cur_filename] = {'prediction': {}}
                    input_data[key]['runs'][cur_filename]['pred_id'] = result[6]
                    input_data[key]['runs'][cur_filename]['se_score'] = result[7]
                    input_data[key]['runs'][cur_filename]['comet_score'] = round(result[8],2) if result[8] else None
                input_data[key]['runs'][cur_filename]['prediction'][result[0]] =  {"error_type": result[1], "error_scale": result[2], "error_explanation": result[3], "color": 'red' if result[2] == 'Major' else 'orange'} if result[1] else "None"
                if does_search_have_error:
                    if result[10] in pred_text_search_ids:
                        input_data[key]['runs'][cur_filename]['prediction'][result[0]]['color'] = 'blue'
                
        
        # Calculate total number of pages
        total_pages = (total_items + load_items_per_page - 1) // load_items_per_page
        help_text = help_text_json["visualize_instruct"]
        
        print(f"search_options: {search_options}")
        
            
        return render_template('visualize_instruct.j2', input_data=input_data, help_text=help_text, total_pages=total_pages, current_page=page_number, files=files, search_options=search_options, search_texts=search_texts, conjunctions=conjunctions)
    
    def get_filename(run_id):
        if run_id not in run_id_dict:
            if len(run_id_dict) == 0:
                results = read_data(f"SELECT id, filename FROM runs;")
            else:
                results = read_data(f"SELECT id, filename FROM runs WHERE id not in {tuple(run_id_dict.keys())}")
            for result in results:
                run_id_dict[result[0]] = result[1]
                
        return run_id_dict[int(run_id)]
    
    def construct_full_query(search_options, search_texts, conjunctions):
        search_query = "SELECT DISTINCT preds.id FROM preds"
        if 'preds_text.error_type' in search_options or 'preds_text.error_scale' in search_options or 'preds_text.error_explanation' in search_options:
            search_query += " JOIN preds_text ON (preds_text.pred_id = preds.id)"
        if 'runs.filename' in search_options:
            search_query += " JOIN runs ON (preds.run_id = runs.id)"
        if 'refs.source_text' in search_options or 'refs.lang' in search_options:
            search_query += " LEFT JOIN refs ON (refs.id = preds.ref_id)"
        if 'src.source_text' in search_options or 'src.lang' in search_options:
            search_query += " LEFT JOIN src ON (src.id = preds.src_id)"
            
        
        is_last_conjunctor_not = False    
        for i, (search_option, search_text) in enumerate(zip(search_options, search_texts)):
            if i > 0:
                if conjunctions[i-1] == 'NOT':
                    search_query += f" AND preds.id NOT IN (SELECT preds.id FROM preds JOIN preds_text ON (preds_text.pred_id = preds.id) AND {search_option} LIKE '%{search_text}%')"
                    is_last_conjunctor_not = True
                else: 
                    search_query += f" {conjunctions[i-1]}"
            else:
                search_query += " WHERE"
            if not is_last_conjunctor_not:
                search_query += get_search_query(search_option, search_text)
            else:
                is_last_conjunctor_not = False
        return search_query
    
    def construct_pred_text_query(search_options, search_texts, conjunctions, pred_ids):
        search_option_errors, search_query_errors, conjunction_errors = trim_search_query_for_error(search_options, search_texts, conjunctions)
        
        search_query = "SELECT DISTINCT preds_text.id FROM preds_text"
        is_last_conjunctor_not = False    
        for i, (search_option, search_text) in enumerate(zip(search_option_errors, search_query_errors)):
            if i > 0:
                if conjunction_errors[i-1] == 'NOT':
                    search_query += f" AND preds_text.id NOT IN (SELECT preds_text.id FROM preds_text WHERE {search_option} LIKE '%{search_text}%')"
                    is_last_conjunctor_not = True
                else: 
                    search_query += f" {conjunction_errors[i-1]}"
            else:
                search_query += " WHERE"
            if not is_last_conjunctor_not:
                search_query += get_search_query(search_option, search_text)
            else:
                is_last_conjunctor_not = False
        
        search_query += f" AND preds_text.pred_id IN {pred_ids};"
        return search_query
        
    def trim_search_query_for_error(search_option, search_query, conjunctions):
        search_option_errors = []
        search_query_errors = []
        conjunction_errors = []
        for i, option in enumerate(search_option):
            if 'error' in option:
                search_option_errors.append(option)
                search_query_errors.append(search_query[i])
                conjunction_errors.append(conjunctions[i] if i < len(conjunctions) else 'AND')
        return search_option_errors, search_query_errors, conjunction_errors
    
    def get_search_query(search_option, search_text):
        return f" {search_option} LIKE '%{search_text}%'"
    
    
    @app.route('/log', methods=['POST'])
    def log_ranking():
        now = datetime.now()
        dt_string = now.strftime("%d/%m/%Y %H:%M:%S")
        try: 
            if isinstance(request.data, bytes):
                data_str = request.data.decode('utf-8')
                data = json.loads(data_str)
            else:
                data = request.get_json()

            
            
            higher_pred_id = data.get('higherPredId')
            lower_pred_id = data.get('lowerPredId')
            
            with open(ranking_log, 'a') as f:
                f.write(f"{dt_string}: {higher_pred_id} > {lower_pred_id}\n")
            
            return jsonify({"status": "success", "message": "Log entry created"}), 200
    
        except Exception as e:
            return jsonify({"status": "error", "message": str(e)}), 400
        
    @app.route('/delete_runs', methods=['POST'])
    def delete_runs():
        try: 
            if isinstance(request.data, bytes):
                data_str = request.data.decode('utf-8')
                data = json.loads(data_str)
            else:
                data = request.get_json()
            run_ids = [int(run_id) for run_id in data]
            
            # start a new process to delete the runs because database connection is in read mode currently
            process = multiprocessing.Process(target=_delete_runs_subprocess, args=(run_ids,))
            process.start()
            process.join()
            
            return jsonify({"status": "success", "message": f"Runs {run_ids} were deleted"}), 200
            
        except Exception as e:
            print(f"Error: {e}")
            return jsonify({"status": "error", "message": str(e)}), 400
    
    @app.route('/clear_search_cache', methods=['POST'])
    def clear_search_cache():
        print(f"session before clearing: {session}")
        if 'search_options' in session:
            del session['search_options']
        if 'search_texts' in session:
            del session['search_texts']
        if 'search_query' in session:
            del session['search_query']
        if 'conjunctions' in session:
            del session['conjunctions']
        print(f"session after clearing: {session}")
        return jsonify({"status": "success", "message": "Search cache cleared"}), 200


    return app


import json
import ast
import itertools

data = json.load(open("reward_data/final_data_scores.json"))
total = 0
# count different types of errors
num_repeat = 0
explanation_consistency_err = 0
err_loc_err = 0
inconsistency_err_loc = 0
inconsistency_err_type = 0
mis_align_err = 0
output_hallucinations = 0
ref_hallucinations = 0
ref_multiple_err = 0
num_label_conflict = 0
num_no_err_minor_disagreement = 0
num_no_err_major_disagreement = 0
num_minor_major_disagreement = 0
multiple_err_loc = 0
total_instances = 0
total_better_ls, total_worse_ls = [], []

for index, sen_dict in data.items():
    sen_score_ls = []
    for sen_index, feedback_dict in sen_dict.items():
        total_instances += 1
        if feedback_dict["no_error_ind"]:
            final_score = feedback_dict["no_error_score"]
        else:
            # first check the repetition
            neg_score = 0
            if feedback_dict["Q7"].split(",")[0] == "Yes":
                num_repeat += 1
                neg_score = int(feedback_dict["Q7"].split(",")[1].strip()) * 4

            cur_err_dict = {}
            for err_index, (err_key, err_dict) in enumerate(feedback_dict.items()):
                if err_key[:3] == "Err":
                    cur_err_dict[err_key] = {
                        "err_type": 1,
                        "err_loc": 1,
                        "major_minor": 1,
                        "explanation": 1,
                    }
                    total += 1
                    """Check all for major/minor labels"""
                    if feedback_dict["labels"][err_index] != err_dict["Q4"]:
                        cur_err_dict[err_key]["major_minor"] = 0
                        cur_err_dict[err_key]["explanation"] = 0
                        if (
                            err_dict["Q4"] == "no-error"
                            and feedback_dict["labels"][err_index] == "minor-error"
                        ):
                            num_no_err_minor_disagreement += 1
                            cur_err_dict[err_key]["err_type"] = 0
                            cur_err_dict[err_key]["err_loc"] = 0
                        elif (
                            err_dict["Q4"] == "no-error"
                            and feedback_dict["labels"][err_index] == "major-error"
                        ):
                            cur_err_dict[err_key]["err_type"] = 0
                            cur_err_dict[err_key]["err_loc"] = 0
                            num_no_err_major_disagreement += 1
                        else:
                            num_minor_major_disagreement += 1

                    """Check all for Error locations"""
                    # check hallucination cases on error location
                    if err_dict["Q1"] != None or err_dict["Q1"] != "None":
                        if isinstance(err_dict["Q1"], list):
                            if (
                                err_dict["Q1"][0] != None
                                and err_dict["Q1"][0] != "None"
                                and (err_dict["Q1"][0] not in feedback_dict["out"])
                            ):
                                err_loc_err += 1
                                cur_err_dict[err_key]["err_loc"] = 0
                        elif isinstance(err_dict["Q1"], str):
                            if (
                                err_dict["Q1"] != "None"
                                and err_dict["Q1"] not in feedback_dict["out"]
                            ):
                                err_loc_err += 1
                                cur_err_dict[err_key]["err_loc"] = 0

                    """Check all for Explanation"""
                    # check the misalignment between two phrases
                    if err_dict["Q3"] == "No":
                        mis_align_err += 1
                        cur_err_dict[err_key]["explanation"] = 0

                    # check the consistency between error location and explanation
                    print(err_dict["Q6"])
                    if err_dict["Q6"] != "Yes":
                        inconsistency_err_loc += 1
                        cur_err_dict[err_key]["err_loc"] = 0

                    # check the consistency between error type and explanation
                    if err_dict["Q5"] != "Yes":
                        inconsistency_err_type += 1
                        cur_err_dict[err_key]["err_type"] = 0

                    if not isinstance(err_dict["Q2"], list):
                        try:
                            err_dict["Q2"] = ast.literal_eval(err_dict["Q2"])
                        except Exception as e:
                            if (
                                err_dict["Q2"].count("[") == 1
                                and err_dict["Q2"].count("]") == 1
                                and err_dict["Q2"].count(", ") == 1
                            ):
                                err_dict["Q2"] = err_dict["Q2"][1:-1].split(", ")
                            else:
                                # All those cases can be considered as multiple errors skip all later lines
                                multiple_err_loc += 1
                                cur_err_dict[err_key]["err_loc"] = 0
                                cur_err_dict[err_key]["explanation"] = 0
                                continue

                    # check output hallucinations in the explanations
                    if err_dict["Q2"] != None and err_dict["Q2"] != "None":
                        if isinstance(err_dict["Q2"][0], list):
                            # check if there are multiple errors in one explanation
                            multiple_err_loc += 1
                            cur_err_dict[err_key]["err_loc"] = 0
                            cur_err_dict[err_key]["explanation"] = 0
                        else:
                            # check the hallucination on the output in explanation
                            if err_dict["Q2"][0] != None and (
                                err_dict["Q2"][0] != "None"
                                and err_dict["Q2"][0] not in feedback_dict["out"]
                            ):
                                output_hallucinations += 1
                                cur_err_dict[err_key]["explanation"] = 0

                            # check the consistency of the explanation
                            if (err_dict["Q2"][0] == err_dict["Q2"][1]) or (
                                err_dict["Q2"][1] != None
                                and isinstance(err_dict["Q2"], str)
                                and err_dict["Q2"][1] in feedback_dict["out"]
                            ):
                                explanation_consistency_err += 1
                                cur_err_dict[err_key]["explanation"] = 0

            cur_score = 0
            total_score = len(cur_err_dict) * 4
            for err_key, score_dict in cur_err_dict.items():
                cur_score += sum(list(score_dict.values()))
            final_score = (cur_score - neg_score) / total_score
        sen_score_ls += [[feedback_dict["prompt"], final_score]]
    better_ls, worse_ls = [], []
    for pair in list(itertools.combinations(sen_score_ls, 2)):
        if pair[0][1] > pair[1][1]:
            better_ls += [pair[0][0]]
            worse_ls += [pair[1][0]]
        elif pair[0][1] < pair[1][1]:
            better_ls += [pair[1][0]]
            worse_ls += [pair[0][0]]
    total_better_ls += better_ls
    total_worse_ls += worse_ls

print("Better: ", len(total_better_ls))
print("Worse: ", len(total_worse_ls))
save_final_dict = {
    "type": "text2text",
    "instances": [
        {"response_j": ele_better, "response_k": ele_worse}
        for ele_better, ele_worse in zip(total_better_ls, total_worse_ls)
    ],
}
with open("final_pairwise_data.json", "w") as f:
    json.dump(save_final_dict, f)
print("File is saved!")
print("Error location hallucination error: ", err_loc_err / total)
print("Error location inconsistency error: ", inconsistency_err_loc / total)
print("Error type inconsistency error: ", inconsistency_err_type / total)
print(
    "Major vs minor disagreement errors in explanation: ",
    num_minor_major_disagreement / total,
)
print(
    "no-error vs minor disagreement errors in explanation: ",
    num_no_err_minor_disagreement / total,
)
print(
    "no-error vs major disagreement errors in explanation: ",
    num_no_err_major_disagreement / total,
)
print("Misligned Error Rate: ", mis_align_err / total)
print("Explanation Output hallucinations: ", output_hallucinations / total)
print("Explanation & Error Locations Multiple errors: ", multiple_err_loc / total)
print("Explanation non-consistency error: ", explanation_consistency_err / total)
print("Repeat percentage: ", num_repeat / total_instances)
print("Total Instance: ", total_instances)
print("Total Errors: ", total)



import glob
from openai.error import RateLimitError
import backoff
import openai
from tqdm.auto import tqdm
import click
import asyncio
from typing import Any


@backoff.on_exception(backoff.expo, RateLimitError)
async def dispatch_openai_requests(
    messages_list,
    model,
    temperature,
    max_tokens,
    top_p,
):
    """Dispatches requests to OpenAI API asynchronously.

    Args:
        messages_list: List of messages to be sent to OpenAI ChatCompletion API.
        model: OpenAI model to use.
        temperature: Temperature to use for the model.
        max_tokens: Maximum number of tokens to generate.
        top_p: Top p to use for the model.
    Returns:
        List of responses from OpenAI API.
    """
    async_responses = [
        openai.ChatCompletion.acreate(
            model=model,
            messages=x,
            temperature=temperature,
            max_tokens=max_tokens,
            top_p=top_p,
        )
        for x in messages_list
    ]
    return await asyncio.gather(*async_responses)


suffix_1 = 'You are evaluating Chinese-to-English Machine translation task. The correct translation is "'
suffix_2 = '". The model generated translation is "'
suffix_3 = '". Please identify all errors'
suffix_4 = "while minor errors don't lead to loss of meaning but will be noticed."

# load in all sample data
cur_dict = {}
for file_name in sorted(glob.glob("data/wmt_samples/*")):
    lines = open(file_name, "r").readlines()
    final_str = "".join(lines)
    samples_ls = final_str.split("[SEP_WENDA]")[:-1]
    for ele in samples_ls:
        if ele.split("\t")[0] not in cur_dict:
            cur_dict[ele.split("\t")[0]] = []
        cur_dict[ele.split("\t")[0]] += [ele.split("\t")[1]]

dist_dict = {}
for index, output in cur_dict.items():
    if len(set(output)) not in dist_dict:
        dist_dict[len(set(output))] = 0
    dist_dict[len(set(output))] += 1


@click.command()
@click.option("-gpt_mode")
@click.option("-api_key")
@click.option("-interval", help="Use 5", type=int)
@click.option("-start_index", help="Use 0", type=int)
@click.option("-end_index", help="Use 400", type=int)
def main(gpt_mode, api_key, interval, start_index, end_index):
    openai.api_key = api_key
    temperature = 0
    count = 0
    max_tokens = 512
    saveFile = open(f"fixed_new_reward_samples_{start_index}_{end_index}.txt", "w")
    saveIndFile = open(
        f"fixed_new_reward_samples_indicater_{start_index}_{end_index}.txt", "w"
    )
    # display all the model outputs + explanations
    index_ls = list(range(0, 10000, interval))[start_index:end_index]
    with tqdm(total=len(index_ls)) as pbar:
        for cur_index in index_ls:
            batch_messages_ls, batch_sample_index_ls, batch_indicater_ls = [], [], []
            for sample_index, ele in enumerate(cur_dict[str(cur_index)]):
                if suffix_4 in ele:
                    score = ele.split(suffix_4)[1].count("Major/minor: Major") * (
                        -5
                    ) + ele.split(suffix_4)[1].count("Major/minor: Minor") * (-1)
                    ref = ele.split(suffix_1)[1].split(suffix_2)[0]
                    out = ele.split(suffix_2)[1].split(suffix_3)[0]
                    if score == 0:
                        # detect if translation is truly no-error
                        sanity_tp_prompt = f"""Reference: {ref}\n Output: {out}\n Compared to the reference, does the output contain any error? (Answer in Yes/No)\n"""
                        batch_messages_ls += [
                            [{"role": "user", "content": sanity_tp_prompt}]
                        ]
                        batch_sample_index_ls += [sample_index]
                        batch_indicater_ls += [True]
                    else:
                        # construct global prompt, to detect overlap or repetition explanations
                        num_errs = int(
                            ele.split(suffix_4)[1]
                            .split("Your Translation contains ")[1]
                            .split("error")[0]
                        )
                        reward_prompt = ""
                        if (
                            f"Error location {num_errs}: " in ele.split(suffix_4)[1]
                            and f"Explanation for error {num_errs}: "
                            in ele.split(suffix_4)[1]
                        ):
                            for i in range(1, 1 + num_errs):
                                err_loc = (
                                    ele.split(suffix_4)[1]
                                    .split(f"Error location {i}: ")[1]
                                    .split("\n")[0]
                                )
                                err_type = (
                                    ele.split(suffix_4)[1]
                                    .split(f"Error type {i}: ")[1]
                                    .split("\n")[0]
                                )
                                reward_prompt += f"Error{i}:\n"
                                reward_prompt += f"Error location {i}: {err_loc}\n"
                                reward_prompt += f"Error type {i}: {err_type}\n"

                                if i < num_errs:
                                    explain = (
                                        ele.split(suffix_4)[1]
                                        .split(f"Explanation for error {i}: ")[1]
                                        .split("\n")[0]
                                    )
                                else:
                                    explain = ele.split(suffix_4)[1].split(
                                        f"Explanation for error {i}: "
                                    )[1]
                                reward_prompt += f"Explanation{i}: {explain}\n"
                            reward_prompt += """\nQ1: For each error location, extract the incorrect error location.\nA1:\n"""
                            reward_prompt += """Q2: Parse explanation into either one of the four forms: [incorrect phrase, correct phrase], [incorrect phrase, None], [None, correct phrase] or [None, None].\nA2:\n"""
                            reward_prompt += f"""\nRef: {ref}\nOut: {out}\n"""
                            reward_prompt += """\nQ3: If A2 is "incorrect phrase to correct phrase", is A2 a correct alignment for reference and output? (Answer Yes/No) If A2 contains None, Answer None.\nA3:\n"""
                            reward_prompt += """Q4: If A2 is "incorrect phrase to correct phrase" and incorrect and correct phrases can be used interchangeably, it is no-error. If an error can be fixed by reading the rest of the output context it is a minor-error. If it changes the sentence's meaning (Don't consider sentence formality changes) and is unable to be fixed by reading the rest of the output it is a major-error. Answer, no-error or minor-error or major-error\nA4:\n"""
                            reward_prompt += """Q5: Is explanation consistent to the given error type?\n"""
                            reward_prompt += """A5: \n"""
                            reward_prompt += """Q6: Does the explanation talk about the given error location?\n"""
                            reward_prompt += """A6: \n"""
                            reward_prompt += """\nQ7: Do two error locations mention the same location in two different ways? (Answer Yes/No) How many pairs of repetitions? (Answer in a number)\nA7: Yes/No, number\n"""
                            reward_prompt += (
                                """\nThe output format will be in JSON\n{"""
                            )
                            cur_ls = []
                            for i in range(1, 1 + num_errs):
                                cur_str = f"Err{i}: "
                                cur_str += (
                                    "{Q1: A1, Q2: A2, Q3: A3, Q4: A4, Q5: A5, Q6: A6}"
                                )
                                cur_ls += [cur_str]
                            cur_ls += ["Q7: A7"]
                            reward_prompt += "{" + (", ").join(cur_ls) + "}"
                            batch_messages_ls += [
                                [{"role": "user", "content": reward_prompt}]
                            ]
                            batch_sample_index_ls += [sample_index]
                            batch_indicater_ls += [False]
                        else:
                            count += 1
                else:
                    count += 1

            try:
                predictions = asyncio.run(
                    dispatch_openai_requests(
                        messages_list=batch_messages_ls,
                        model=gpt_mode,
                        temperature=temperature,
                        max_tokens=max_tokens,
                        top_p=1.0,
                    )
                )

                for pred, sample_index, cur_sample_indicater in zip(
                    predictions, batch_sample_index_ls, batch_indicater_ls
                ):
                    saveFile.write(
                        str(cur_index)
                        + "\t"
                        + str(sample_index)
                        + "\t"
                        + pred["choices"][0]["message"]["content"]
                        + "[SEP_TOK_WENDA]"
                    )
                    saveFile.flush()
                    saveIndFile.write(
                        str(cur_index)
                        + "\t"
                        + str(sample_index)
                        + "\t"
                        + str(cur_sample_indicater)
                        + "[SEP_TOK_WENDA]"
                    )
                    saveIndFile.flush()

            except Exception as e:
                print(f"ERROR: {e}")
            pbar.update(1)

    print(count)
    print("All files are saved!")


if __name__ == "__main__":
    main()



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional, Union, Sequence
import evaluate
import numpy as np
import torch
import torch.nn as nn
from datasets import load_dataset
from transformers import (
    AutoConfig,
    AutoModelForSequenceClassification,
    AutoTokenizer,
    HfArgumentParser,
    PreTrainedTokenizerBase,
    Trainer,
    TrainingArguments,
)
from transformers.utils import PaddingStrategy
from torch.utils.data import Dataset
import transformers
import copy
from tqdm.auto import tqdm
from peft import LoraConfig, TaskType, get_peft_model

DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"
max_length = 720
padding_strategy = "right"
IGNORE_INDEX = -100
output_dir = "reward_model_weights"


class SupervisedDataset(Dataset):
    """Dataset for supervised fine-tuning."""

    def __init__(self, tokenizer: transformers.PreTrainedTokenizer):
        super(SupervisedDataset, self).__init__()
        data_dict = preprocess(raw_dataset, tokenizer)
        self.input_ids_j = data_dict["input_ids_j"]
        self.input_ids_k = data_dict["input_ids_k"]
        self.attention_mask_j = data_dict["attention_mask_j"]
        self.attention_mask_k = data_dict["attention_mask_k"]

    def __len__(self):
        return len(self.input_ids_j)

    def __getitem__(self, i) -> Dict[str, torch.Tensor]:
        return dict(
            input_ids_j=self.input_ids_j[i],
            attention_mask_j=self.attention_mask_j[i],
            input_ids_k=self.input_ids_k[i],
            attention_mask_k=self.attention_mask_k[i],
        )


def make_supervised_data_module(tokenizer: transformers.PreTrainedTokenizer) -> Dict:
    """Make dataset and collator for supervised fine-tuning."""
    train_dataset = SupervisedDataset(tokenizer=tokenizer)
    data_collator = DataCollatorForSupervisedDataset(tokenizer=tokenizer)
    return dict(
        train_dataset=train_dataset, eval_dataset=None, data_collator=data_collator
    )


def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
    model: transformers.PreTrainedModel,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    num_new_tokens = tokenizer.add_special_tokens(special_tokens_dict)
    model.resize_token_embeddings(len(tokenizer))

    if num_new_tokens > 0:
        input_embeddings = model.get_input_embeddings().weight.data
        input_embeddings_avg = input_embeddings[:-num_new_tokens].mean(
            dim=0, keepdim=True
        )
        input_embeddings[-num_new_tokens:] = input_embeddings_avg


# Turn the dataset into pairs of post + summaries, where text_j is the preferred question + answer and text_k is the other.
# Then tokenize the dataset.
def preprocess(examples, tokenizer):
    new_examples = {
        "input_ids_j": [],
        "attention_mask_j": [],
        "input_ids_k": [],
        "attention_mask_k": [],
    }
    # with tqdm(total=len(examples["response_j"])) as pbar:
    final_dict = _tokenize_fn(examples["response_j"], examples["response_k"], tokenizer)

    new_examples["input_ids_j"] = final_dict["input_ids_j"]
    new_examples["attention_mask_j"] = final_dict["attention_mask_j"]
    new_examples["input_ids_k"] = final_dict["input_ids_k"]
    new_examples["attention_mask_k"] = final_dict["attention_mask_k"]
    # pbar.update(1)
    return new_examples


# We need to define a special data collator that batches the data in our j vs k format.
def _tokenize_fn(
    strings_j: Sequence[str],
    strings_k: Sequence[str],
    tokenizer: transformers.PreTrainedTokenizer,
) -> Dict:
    """Tokenize a list of strings."""
    tokenized_list_j = [
        tokenizer(
            text,
            return_tensors="pt",
            padding="max_length",
            max_length=max_length,
            truncation=True,
        )
        for text in strings_j
    ]
    print("Batch J responses are tokenized")
    tokenized_list_k = [
        tokenizer(
            text,
            return_tensors="pt",
            padding="max_length",
            max_length=max_length,
            truncation=True,
        )
        for text in strings_k
    ]
    print("Bacth K responses are tokenized!")
    input_ids_j = [tokenized.input_ids[0] for tokenized in tokenized_list_j]
    input_ids_k = [tokenized.input_ids[0] for tokenized in tokenized_list_k]
    attention_mask_j = [tokenized.attention_mask[0] for tokenized in tokenized_list_j]
    attention_mask_k = [tokenized.attention_mask[0] for tokenized in tokenized_list_k]
    print("All tokenizations are done")
    return dict(
        input_ids_j=input_ids_j,
        input_ids_k=input_ids_k,
        attention_mask_j=attention_mask_j,
        attention_mask_k=attention_mask_k,
        return_loss=True,
    )


@dataclass
class DataCollatorForSupervisedDataset(object):
    """Collate examples for supervised fine-tuning."""

    tokenizer: transformers.PreTrainedTokenizer

    def __call__(self, instances: Sequence[Dict]) -> Dict[str, torch.Tensor]:
        # print("check instance")
        # print(instances)
        input_ids_j = [instance["input_ids_j"] for instance in instances]
        input_ids_j = torch.nn.utils.rnn.pad_sequence(
            input_ids_j, batch_first=True, padding_value=self.tokenizer.pad_token_id
        )
        input_ids_k = [instance["input_ids_k"] for instance in instances]
        input_ids_k = torch.nn.utils.rnn.pad_sequence(
            input_ids_k, batch_first=True, padding_value=self.tokenizer.pad_token_id
        )

        return dict(
            input_ids_j=input_ids_j,
            attention_mask_j=input_ids_j.ne(self.tokenizer.pad_token_id),
            input_ids_k=input_ids_k,
            attention_mask_k=input_ids_k.ne(self.tokenizer.pad_token_id),
            return_loss=True,
        )


class RewardTrainer(Trainer):
    # Define how to compute the reward loss. We use the InstructGPT pairwise logloss: https://arxiv.org/abs/2203.02155
    def compute_loss(self, model, inputs, return_outputs=False):
        rewards_j = model(
            input_ids=inputs["input_ids_j"], attention_mask=inputs["attention_mask_j"]
        )[0]
        rewards_k = model(
            input_ids=inputs["input_ids_k"], attention_mask=inputs["attention_mask_k"]
        )[0]
        loss = -nn.functional.logsigmoid(rewards_j - rewards_k).mean()
        if return_outputs:
            return loss, {"rewards_j": rewards_j, "rewards_k": rewards_k}
        return loss


# Load the dataset using the HuggingFace dataset library
extensions = "json"
KEY_INSTANCES = "instances"
raw_dataset = load_dataset(
    extensions,
    data_files=["final_pairwise_data.json"],
    field=KEY_INSTANCES,
    split="train",
    use_auth_token=None,
)

print(len(raw_dataset["response_j"]))
print(len(raw_dataset["response_k"]))

# Load the value-head model and tokenizer.
tokenizer = AutoTokenizer.from_pretrained(
    "decapoda-research/llama-7b-hf",
    model_max_length=max_length,
    padding_side=padding_strategy,
    use_fast=False,
)

peft_config = LoraConfig(
    task_type=TaskType.SEQ_CLS,
    inference_mode=False,
    r=8,
    lora_alpha=32,
    lora_dropout=0.1,
)
model = AutoModelForSequenceClassification.from_pretrained(
    "decapoda-research/llama-7b-hf", num_labels=1, torch_dtype=torch.bfloat16
)
# model = get_peft_model(model, peft_config)
# model.print_trainable_parameters()

print("Loaded in model and tokenizers")
if tokenizer.pad_token is None:
    smart_tokenizer_and_embedding_resize(
        special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
        tokenizer=tokenizer,
        model=model,
    )

tokenizer.add_special_tokens(
    {
        "eos_token": DEFAULT_EOS_TOKEN,
        "bos_token": DEFAULT_BOS_TOKEN,
        "unk_token": DEFAULT_UNK_TOKEN,
    }
)

ds_config = "config/ds_config_zero3.json"
data_module = make_supervised_data_module(tokenizer=tokenizer)
training_args = TrainingArguments(
    output_dir=output_dir,
    evaluation_strategy="no",
    per_device_train_batch_size=1,
    gradient_accumulation_steps=32,
    learning_rate=2e-5,
    weight_decay=0,
    num_train_epochs=3,
    warmup_ratio=0,
    logging_strategy="steps",
    logging_first_step=True,
    save_strategy="epoch",
    save_total_limit=3,
    seed=42,
    run_name="wandb",
    load_best_model_at_end=False,
    greater_is_better=False,
    deepspeed=ds_config,
    log_on_each_node=False,
    logging_steps=1,
    fp16=True,
    lr_scheduler_type="cosine",
)

# Train the model
trainer = RewardTrainer(
    model=model,
    args=training_args,
    train_dataset=data_module["train_dataset"],
    eval_dataset=None,
    tokenizer=tokenizer,
    data_collator=data_module["data_collator"],
    preprocess_logits_for_metrics=None,
)

trainer.train()
print("Saving last checkpoint of the model")
model.save_pretrained(output_dir)



from transformers import AutoTokenizer, pipeline
from accelerate import Accelerator
import json
from typing import Dict
import transformers
import click
import glob

final_dict = {}
for file_addr in glob.glob("test_wmt22_zh-en/SEScore3_output_sample_222/*"):
    file_name = file_addr.split("/")[-1]
    sys_name = file_name.split("test_wmt22_zh-en_")[1].split("_llama_")[0]
    final_str = ""
    lines = open(file_addr, "r").readlines()
    for ele in lines:
        final_str += ele
    if sys_name not in final_dict:
        final_dict[sys_name] = {}
    for cur_str in final_str.split("[SEP_WENDA]")[:-1]:
        sen_index, ret_index, output = (
            cur_str.split("\t")[0],
            cur_str.split("\t")[1],
            cur_str.split("\t")[2],
        )
        if sen_index not in final_dict[sys_name]:
            final_dict[sys_name][sen_index] = {}
        final_dict[sys_name][sen_index][ret_index] = output

# sys_ls = [
#     "LanguageX",
#     "M2M100_1.2B-B4",
#     "Online-A",
#     "Online-B",
#     "Online-G",
#     "Online-W",
#     "Online-Y",
#     "bleurt_bestmbr",
# ]

DEFAULT_PAD_TOKEN = "[PAD]"
DEFAULT_EOS_TOKEN = "</s>"
DEFAULT_BOS_TOKEN = "</s>"
DEFAULT_UNK_TOKEN = "</s>"


def smart_tokenizer_and_embedding_resize(
    special_tokens_dict: Dict,
    tokenizer: transformers.PreTrainedTokenizer,
):
    """Resize tokenizer and embedding.
    Note: This is the unoptimized version that may make your embedding size not be divisible by 64.
    """
    tokenizer.add_special_tokens(special_tokens_dict)
    tokenizer.add_special_tokens(
        {
            "eos_token": DEFAULT_EOS_TOKEN,
            "bos_token": DEFAULT_BOS_TOKEN,
            "unk_token": DEFAULT_UNK_TOKEN,
        }
    )


current_device = Accelerator().local_process_index
sent_kwargs = {
    "return_all_scores": True,
    "function_to_apply": "none",
    "batch_size": 6,
    "truncation": True,
}


@click.command()
@click.option("-sys_name", type=str)
def main(sys_name):
    reward_model_name = f"/share/edc/home/wendaxu/Reward_Model"
    tokenizer = AutoTokenizer.from_pretrained("decapoda-research/llama-7b-hf")

    smart_tokenizer_and_embedding_resize(
        special_tokens_dict=dict(pad_token=DEFAULT_PAD_TOKEN),
        tokenizer=tokenizer,
    )

    sentiment_pipe = pipeline(
        "sentiment-analysis",
        model=reward_model_name,
        device_map={"": current_device},
        tokenizer=tokenizer,
    )

    output_ls = []
    for ele in list(final_dict[sys_name].values()):
        output_ls += [cur_ele for cur_ele in ele.values()]

    pipe_outputs = sentiment_pipe(output_ls, **sent_kwargs)
    with open(f"save_{sys_name}_scores_3.json", "w") as f:
        json.dump({f"{sys_name}": [ele[0]["score"] for ele in pipe_outputs]}, f)


if __name__ == "__main__":
    main()



import json
from transformers import LlamaTokenizer
import click

tokenizer = LlamaTokenizer.from_pretrained("decapoda-research/llama-7b-hf")

@click.command()
@click.option('-src_ref')
def main(src_ref):
    data = json.load(open(f'filter_{src_ref}_based_data_april_6.json'))
    final_dict = {'type': "text2text", "instances": []}
    len_ls = []
    for _, cur_data in data.items():
        cur_len=len(tokenizer.tokenize(cur_data['input']+cur_data['output']))
        len_ls+=[cur_len]
        if cur_len>682:
            print(cur_len)
        final_dict["instances"]+=[cur_data]

    print(max(len_ls))
    with open(f'llama_{src_ref}_data.json', 'w') as f:
        json.dump(final_dict, f)

if __name__ == "__main__":
    main()



from translation_canvas.file_extraction_scripts.extract_pairs_from_log import extract_pairs_from_log
from translation_canvas.file_extraction_scripts.extract_pairs_from_log import extract_pairs_from_log
from translation_canvas.file_extraction_scripts.extract_pairs_from_json import extract_pairs_from_json
from translation_canvas.file_extraction_scripts.extract_pairs_from_csv import extract_pairs_from_csv
from translation_canvas.file_extraction_scripts.extract_pairs_from_tsv import extract_pairs_from_tsv
from translation_canvas.file_extraction_scripts.extract_pairs_from_xml import extract_pairs_from_xml
from translation_canvas.file_extraction_scripts.extract_pairs_from_txt import extract_pairs_from_txt
from argparse import ArgumentParser
import os
import json

JOBS = os.path.join(os.path.dirname(__file__), 'jobs')
if not os.path.exists(JOBS):
    os.mkdir(JOBS)

parser = ArgumentParser()
parser.add_argument('--file_names', type=str, default="")
parser.add_argument('--memorable_name', type=str, default="")
parser.add_argument('--file_type', type=str, default="")

args = parser.parse_args()

if args.file_type == "log":
    pairs = extract_pairs_from_log(args.file_names)
elif args.file_type == "json":
    pairs = extract_pairs_from_json(args.file_names)
elif args.file_type == "csv":
    pairs = extract_pairs_from_csv(args.file_names)
elif args.file_type == "tsv":
    pairs = extract_pairs_from_tsv(args.file_names)
elif args.file_type == "xml":
    pairs = extract_pairs_from_xml(args.file_names)
else:
    pairs = extract_pairs_from_txt(args.file_names)
    
if not os.path.exists(os.path.join(JOBS, args.memorable_name)):
    os.mkdir(os.path.join(JOBS, args.memorable_name))
    
     
with open(os.path.join(JOBS, args.memorable_name, args.memorable_name + "_extracted.json"), 'w') as f:
    json.dump(pairs, f, indent=4)